@@ -1,18 +1,15 @@
-import types
-# import itertools
-import numpy
 import re
-
+import types
 from enum import Enum, unique
 from functools import reduce
 from itertools import product
 
-from pycsp3.classes.auxiliary.ptypes import auto
-from pycsp3.classes.main.variables import Variable, NotVariable, NegVariable
 from pycsp3.classes import main
-from pycsp3.dashboard import options
-from pycsp3.tools.utilities import flatten, is_containing
+from pycsp3.classes.auxiliary.ptypes import auto
+from pycsp3.classes.main.variables import Variable
 from pycsp3.tools.inspector import checkType
+from pycsp3.tools.utilities import flatten, is_containing
+from pycsp3.classes.auxiliary.ptypes import TypeCtr, TypeConditionOperator
 
 
 class Entity:
@@ -75,7 +72,9 @@ class EVarArray(Entity):
         self.name = name
         self.variables = X
         self.flatVars = flatten(X)
-        assert len(self.flatVars) != 0, "Array of variable empty !"
+        if len(self.flatVars) == 0:
+            return
+        # assert len(self.flatVars) != 0, "Array of variable empty !"
         self.containing_hole = None  # undefined until we ask  #flatVarsKeepingNone = flatten(X, keep_none=True)
         self.size = []
         curr = self.variables
@@ -136,7 +135,7 @@ class ECtrs(Entity):
 
     def __init__(self, constraints=None):
         super().__init__(None)  # no need to have an id here
-        assert constraints is not None and isinstance(constraints, list)
+        assert isinstance(constraints, list)
         self.entities = [c for c in constraints if c is not None]
 
 
@@ -157,6 +156,9 @@ class EToSatisfy(ECtrs):
             CtrEntities.items.append(self)
             super().__init__(constraints)
 
+    def delete(self, i):
+        del self.entities[i]
+
 
 class EGroup(ECtrs):
     ''' Constraints in a group '''
@@ -182,11 +184,49 @@ class ESlide(ECtrs):
         self.circular = False
 
 
-class EIfThenElse(ECtrs):
+class EMetaCtr(Entity):
+    def __init__(self, name, constraints, min_arity, max_arity=None):
+        super().__init__(name)  # no need to have an id here
+        assert isinstance(constraints, list)
+        self.entities = [c for c in constraints if c is not None]
+        checkType(self.entities, [ECtr, EMetaCtr])
+        assert len(self.entities) >= min_arity, "At least " + str(min_arity) + " components must be specified in the meta-constraint"
+        assert max_arity is None or len(self.entities) <= max_arity, "At most " + str(max_arity) + " components must be specified in the meta-constraint"
+
+
+class EAnd(EMetaCtr):
     def __init__(self, constraints):
-        checkType(constraints, [ECtr])
-        assert len(constraints) == 3, "Error: three components must be specified in ifThenElse"
-        super().__init__(constraints)
+        super().__init__(TypeCtr.AND, constraints, 2)
+
+
+class EOr(EMetaCtr):
+    def __init__(self, constraints):
+        super().__init__(TypeCtr.OR, constraints, 2)
+
+
+class ENot(EMetaCtr):
+    def __init__(self, constraints):
+        super().__init__(TypeCtr.NOT, constraints, 1, 1)
+
+
+class EXor(EMetaCtr):
+    def __init__(self, constraints):
+        super().__init__(TypeCtr.XOR, constraints, 2)
+
+
+class EIfThen(EMetaCtr):
+    def __init__(self, constraints):
+        super().__init__(TypeCtr.IF_THEN, constraints, 2, 2)
+
+
+class EIfThenElse(EMetaCtr):
+    def __init__(self, constraints):
+        super().__init__(TypeCtr.IF_THEN_ELSE, constraints, 3, 3)
+
+
+class EIff(EMetaCtr):
+    def __init__(self, constraints):
+        super().__init__(TypeCtr.IFF, constraints, 2, 2)
 
 
 class EObjective(Entity):
@@ -295,6 +335,28 @@ class TypeNode(Enum):
     def is_predicate_operator(self):
         return self.is_logical_operator() or self.is_relational_operator() or self in {TypeNode.IN, TypeNode.NOTIN}
 
+    @staticmethod
+    def value_of(v):
+        if isinstance(v, TypeNode):
+            return v
+        if isinstance(v, str):
+            if v == "<":
+                return TypeNode.LT
+            if v == "<=":
+                return TypeNode.LE
+            if v == ">=":
+                return TypeNode.GE
+            if v == ">":
+                return TypeNode.GT
+            if v in ("=", "=="):
+                return TypeNode.EQ
+            if v in ("!=", "<>"):
+                return TypeNode.NE
+            return TypeNode[v.upper()]
+        if isinstance(v, TypeConditionOperator):
+            return TypeNode[str(v).upper()]
+        return None  # other cases to handle?
+
 
 def neg_range(r):
     assert isinstance(r, range) and r.step == 1
@@ -330,9 +392,22 @@ class Node(Entity):
         self.abstractTree = None
         self.abstractValues = None
 
+    def eq__safe(self, other):
+        if not isinstance(other, Node) or self.type != other.type or self.leaf != other.leaf:
+            return False
+        if not self.leaf:
+            return len(self.sons) == len(other.sons) and all(self.sons[i].eq__safe(other.sons[i]) for i in range(len(self.sons)))
+        return self.sons.eq__safe(other.sons) if isinstance(self.sons, Variable) else self.sons == other.sons
+
     def __str__(self):
         return str(self.sons) if self.type.is_leaf() else str(self.type) + "(" + ",".join(str(son) for son in self.sons) + ")"
 
+    def _product(t):
+        p = 1
+        for i in t:
+            p *= i
+        return p
+
     def possible_values(self):
         if self.type.is_predicate_operator():
             return range(0, 2)  # we use a range instead of [0,1] because it simplifies computation (see code below)
@@ -343,7 +418,7 @@ class Node(Entity):
                     return av
                 return range(av[0], av[0] + 1) if len(av) == 1 else range(av[0], av[1] + 1) if len(av) == 2 and av[0] + 1 == av[1] else av
             if self.type == TypeNode.INT:
-                return range(self.sons, self.sons+1)  # we use a range instead of a singleton list because it simplifies computation (see code below)
+                return range(self.sons, self.sons + 1)  # we use a range instead of a singleton list because it simplifies computation (see code below)
             assert False, "no such 0-ary type " + str(self.type) + " is expected"
         if self.type.min_arity == self.type.max_arity == 1:
             pv = self.sons[0].possible_values()
@@ -379,7 +454,7 @@ class Node(Entity):
             if self.type == TypeNode.ADD:
                 return reduce(add_range, pvs) if all_ranges else possible_range({sum(p) for p in product(*(pv for pv in pvs))})
             if self.type == TypeNode.MUL:
-                return possible_range({numpy.prod(p) for p in product(*(pv for pv in pvs))})
+                return possible_range({self._product(p) for p in product(*(pv for pv in pvs))})  # or numpy.prod ?
             # TODO: in case of all_ranges being False, possibility of improving the efficiency of the code below for MIN and MAX
             if self.type == TypeNode.MIN:
                 return range(min(pv.start for pv in pvs), min(pv.stop for pv in pvs)) if all_ranges \
@@ -488,12 +563,13 @@ class Node(Entity):
                 t.append(arg)
             elif isinstance(arg, EVar):
                 t.append(Node(TypeNode.VAR, arg.variable))
-            elif isinstance(arg, NotVariable):
-                t.append(Node(TypeNode.NOT, [Node(TypeNode.VAR, arg.variable)]))
-            elif isinstance(arg, NegVariable):
-                t.append(Node(TypeNode.NEG, [Node(TypeNode.VAR, arg.variable)]))
             elif isinstance(arg, Variable):
-                t.append(Node(TypeNode.VAR, arg))
+                if arg.inverse:
+                    t.append(Node(TypeNode.NEG, [Node(TypeNode.VAR, arg)]))
+                elif arg.negation:
+                    t.append(Node(TypeNode.NOT, [Node(TypeNode.VAR, arg)]))
+                else:
+                    t.append(Node(TypeNode.VAR, arg))
             elif isinstance(arg, int):
                 t.append(Node(TypeNode.INT, arg))
             elif isinstance(arg, str):
@@ -506,21 +582,7 @@ class Node(Entity):
 
     @staticmethod
     def build(type, *args):
-        if isinstance(type, str):
-            if type == "<":
-                type = TypeNode.LT
-            elif type == "<=":
-                type = TypeNode.LE
-            elif type == ">=":
-                type = TypeNode.GE
-            elif type == ">":
-                type = TypeNode.GT
-            elif type in {"=", "=="}:
-                type = TypeNode.EQ
-            elif type == "!=":
-                type = TypeNode.NE
-            else:
-                type = TypeNode[type.upper()]
+        type = TypeNode.value_of(type)  # for handling the cases where type is of type str or TypeConditionOperator
         if type is TypeNode.SET:
             assert len(args) == 1
             elements = list(args[0])
@@ -528,12 +590,12 @@ class Node(Entity):
             return Node(type, Node._create_sons(*sorted_sons))  # *sorted(args[0])))
         args = flatten(Node.build(TypeNode.SET, arg) if isinstance(arg, (set, range, frozenset)) else arg for arg in args)
         assert type.is_valid_arity(len(args)), "Problem: Bad arity for node " + type.name + ". It is " + str(
-            len(args)) + " but it should be between " + str(type.arityMin) + " and " + str(type.arityMax)
+            len(args)) + " but it should be between " + str(type.min_arity) + " and " + str(type.max_arity)
         node = Node(type, Node._create_sons(*args))
         if type == TypeNode.EQ and all(son.type.is_predicate_operator() for son in node.sons):
             node = Node(TypeNode.IFF, node.sons)
         # Reducing the node
-        for t in {TypeNode.ADD, TypeNode.MUL, TypeNode.OR, TypeNode.AND, TypeNode.EQ, TypeNode.IFF}:
+        for t in {TypeNode.ADD, TypeNode.MUL, TypeNode.OR, TypeNode.AND}:
             node.flatten_by_associativity(t)
         node.reduce_integers()
         return node
@@ -550,6 +612,8 @@ class Node(Entity):
                 args = tuple(args[0])
             if isinstance(args[0], types.GeneratorType):
                 args = tuple(list(args[0]))
+        if len(args) == 0:
+            return True if t == TypeNode.AND else False
         return Node.build(t, *args) if len(args) > 1 else args[0]
 
     @staticmethod
