@@ -1,15 +1,17 @@
 import os
 from collections import OrderedDict
 
+from pycsp3 import functions
 from pycsp3.classes.auxiliary.conditions import Condition, ConditionInterval, ConditionSet
 from pycsp3.classes.auxiliary.ptypes import TypeVar, TypeCtr, TypeCtrArg, TypeXML, TypeConditionOperator, TypeRank
 from pycsp3.classes.auxiliary.values import IntegerEntity
-from pycsp3.classes.entities import EVarArray, ECtr, TypeNode, Node
+from pycsp3.classes.entities import EVarArray, ECtr, EMetaCtr, TypeNode, Node, possible_range
 from pycsp3.classes.main.domains import Domain
 from pycsp3.classes.main.variables import Variable, VariableInteger
-from pycsp3.tools.compactor import compact
-from pycsp3.tools.utilities import ANY, is_1d_list, matrix_to_string, transitions_to_string, integers_to_string, table_to_string, flatten, is_matrix, error, \
-    to_ordinary_table
+from pycsp3.dashboard import options
+from pycsp3.tools import curser
+from pycsp3.tools.utilities import ANY, is_1d_list, matrix_to_string, integers_to_string, table_to_string, flatten, is_matrix, error, \
+    to_ordinary_table, warning, is_windows
 
 
 class Diffs:
@@ -48,8 +50,23 @@ class ConstraintArgument:
         self.content_ordered = content_ordered  # indicates if the content must be kept as it is (order is important)
         self.lifted = lifted  # indicates if the constraint is lifted to several lists (or sets); see specifications
 
+    def __eq__(self, other):  # must be called in protection mode (see in functions.py the function protect())
+        if not isinstance(other, ConstraintArgument) or self.name != other.name or len(self.attributes) > 0 or len(other.attributes) > 0:
+            return False  # attributes not currently completely taken into account
+        if self.content_compressible != other.content_compressible or self.content_ordered != other.content_ordered or self.lifted != other.lifted:
+            return False
+        return curser.OpOverrider.eq_protected(self.content, other.content)
+
+        # #print(type(self.content),type(other.content))
+        # if isinstance(self.content, list):  #
+        #     return list.__eq__(self.content, other.content)
+        # #if isinstance(self.content, (EMetaCtr, Variable, Node)):
+        # #    return object.__eq__(self.content, other.content)
+        # return object.__eq__(self.content, other.content)
+        # return self.content == other.content
+
     def __str__(self):
-        return str(self.name) + str(self.content)
+        return str(self.name) + ":" + str(self.content)
 
 
 class Constraint:
@@ -59,6 +76,20 @@ class Constraint:
         self.arguments = OrderedDict()  # arguments of the constraint (such as list, supports, condition, ...)
         self.n_parameters = 0  # used when building abstract forms of constraints (e.g., in groups for %0 %1 %2 ...)
 
+    def __eq__(self, other):  # must be called in protection mode (see in functions.py the function protect())
+        if not isinstance(other, Constraint) or self.name != other.name or len(self.attributes) > 0 or len(other.attributes) > 0:
+            return False  # attributes not currently completely taken into account
+        if self.n_parameters > 0 or other.n_parameters > 0:
+            return False
+        return self.arguments == other.arguments
+
+    def equal_except_condition(self, other):
+        if not isinstance(other, Constraint) or self.name != other.name or len(self.attributes) > 0 or len(other.attributes) > 0:
+            return False  # attributes not currently completely taken into account
+        if self.n_parameters > 0 or other.n_parameters > 0:
+            return False
+        return [v for v in self.arguments.items() if str(v[0]) != "condition"] == [v for v in other.arguments.items() if str(v[0]) != "condition"]
+
     def arg(self, name, content, *, attributes=[], content_compressible=True, content_ordered=False, lifted=False):
         self.arguments[name] = ConstraintArgument(name, content, attributes, content_compressible, content_ordered, lifted)
         return self
@@ -86,6 +117,8 @@ class Constraint:
         records = []
         for i in range(len(args1)):
             if str(args1[i].content) != str(args2[i].content):
+                if args1[i].name == TypeCtrArg.MATRIX:  # TODO making a more specific test?
+                    return False
                 if args1[i].name == TypeCtrArg.CONDITION and args1[i].content.operator != args2[i].content.operator:
                     return False  # the operators are different in the two conditions
                 b = hasattr(args1[i].content, '__len__') and hasattr(args2[i].content, '__len__') and len(args1[i].content) != len(args2[i].content)
@@ -96,7 +129,7 @@ class Constraint:
         if len(records) == 0:
             return diffs
         if Diffs.fusion and not all(
-                        name in Diffs.fusion.argument_names for name in diffs.argument_names):  # diffs.argument_names != Diffs.fusion.argument_names:
+                name in Diffs.fusion.argument_names for name in diffs.argument_names):  # diffs.argument_names != Diffs.fusion.argument_names:
             return False  # because arguments are not the same (so it is not possible to make a unique abstraction)
         if len([flag for flag in diffs.argument_flags if flag]) > 1:
             return False  # two arguments of different size cannot be currently abstracted
@@ -214,6 +247,7 @@ class ConstraintExtension(Constraint):
                 ConstraintExtension.cache[h] = integers_to_string(table) if isinstance(table[0], int) else " ".join(v for v in sorted(table))
             return ConstraintExtension.cache[h]
 
+        possible_parallelism = not options.safe and not is_windows()
         if h in ConstraintExtension.cache_smart:
             smart = ConstraintExtension.cache_smart[h]
         else:
@@ -225,24 +259,25 @@ class ConstraintExtension(Constraint):
                 if h not in ConstraintExtension.cache:
                     table.sort()
                     table = ConstraintExtension.remove_redundant_tuples(table)
-                    ConstraintExtension.cache[h] = table_to_string(table, parallel=os.name != 'nt')
+                    ConstraintExtension.cache[h] = table_to_string(table, parallel=possible_parallelism)
                 return ConstraintExtension.cache[h]
             else:
                 # if ever we would want to use caching, we should include domains when computing hashes
                 table.sort()
-                return table_to_string(table, restricting_domains=[x.dom for x in scope], parallel=os.name != 'nt')
+                return table_to_string(table, restricting_domains=[x.dom for x in scope], parallel=possible_parallelism)
         else:  # it is smart
             if self.keepsmartconditions:  # currently, no restriction of tables (wrt domains) in that case
                 self.attributes.append((TypeXML.TYPE, "smart"))
                 if h not in ConstraintExtension.cache:
                     table = ConstraintExtension.remove_redundant_tuples(table)
-                    ConstraintExtension.cache[h] = table_to_string(table, parallel=os.name != 'nt')
+                    ConstraintExtension.cache[h] = table_to_string(table, parallel=possible_parallelism)
                 return ConstraintExtension.cache[h]
             else:
                 # if ever we would want to use caching, we should include domains when computing hashes
                 table = ConstraintExtension.convert_smart_to_ordinary(scope, table)
                 table = ConstraintExtension.remove_redundant_tuples(table)
-                return table_to_string(table, restricting_domains=[x.dom for x in scope] if self.restrictTablesWrtDomains else None, parallel=os.name != 'nt')
+                return table_to_string(table, restricting_domains=[x.dom for x in scope] if self.restrictTablesWrtDomains else None,
+                                       parallel=possible_parallelism)
 
     def __init__(self, scope, table, positive=True, keepsmartconditions=False, restrictTablesWrtDomains=False):
         super().__init__(TypeCtr.EXTENSION)
@@ -272,7 +307,7 @@ class ConstraintRegular(Constraint):
     def __init__(self, lst, automaton):
         super().__init__(TypeCtr.REGULAR)
         self.arg(TypeCtrArg.LIST, lst, content_ordered=True)
-        self.arg(TypeCtrArg.TRANSITIONS, transitions_to_string(automaton.transitions))
+        self.arg(TypeCtrArg.TRANSITIONS, automaton.transitions_to_string())
         self.arg(TypeCtrArg.START, automaton.start)
         self.arg(TypeCtrArg.FINAL, automaton.final)
 
@@ -282,14 +317,14 @@ class ConstraintRegular(Constraint):
 
 
 class ConstraintMdd(Constraint):
-    def __init__(self, lst, transitions):
+    def __init__(self, lst, mdd):
         super().__init__(TypeCtr.MDD)
         self.arg(TypeCtrArg.LIST, lst, content_ordered=True)
         # TODO reordering transitions in order to guarantee to have:
         # - the root as the src of the first transition
         # - the terminal as the dst of the last transition
         # - no transition with a src occurring before it was reached
-        self.arg(TypeCtrArg.TRANSITIONS, transitions_to_string(transitions))
+        self.arg(TypeCtrArg.TRANSITIONS, mdd.transitions_to_string())
 
 
 ''' Comparison-based Constraints '''
@@ -358,7 +393,7 @@ class ConstraintSum(ConstraintWithCondition):
         vs = self.arguments[TypeCtrArg.LIST].content
         cs = self.arguments[TypeCtrArg.COEFFS].content if TypeCtrArg.COEFFS in self.arguments else None
         if cs is None:
-            return sum(x.dom.smallest_value() for x in vs)
+            return sum(x.dom.smallest_value() if isinstance(x, Variable) else x.possible_values()[0] for x in vs)
         assert len(vs) == len(cs)
         t = []
         for i, x in enumerate(vs):
@@ -374,7 +409,7 @@ class ConstraintSum(ConstraintWithCondition):
         vs = self.arguments[TypeCtrArg.LIST].content
         cs = self.arguments[TypeCtrArg.COEFFS].content if TypeCtrArg.COEFFS in self.arguments else None
         if cs is None:
-            return sum(x.dom.greatest_value() for x in vs)
+            return sum(x.dom.greatest_value() if isinstance(x, Variable) else x.possible_values()[-1] for x in vs)
         assert len(vs) == len(cs)
         t = []
         for i, x in enumerate(vs):
@@ -386,6 +421,13 @@ class ConstraintSum(ConstraintWithCondition):
                 t.append(max(xmin * cmin, xmin * cmax, xmax * cmin, xmax * cmax))
         return sum(t)
 
+    def revert_coeffs(self):
+        if TypeCtrArg.COEFFS in self.arguments:
+            self.arguments[TypeCtrArg.COEFFS].content = [-v for v in self.arguments[TypeCtrArg.COEFFS].content]
+        else:
+            self.arg(TypeCtrArg.COEFFS, [-1 for _ in range(len(self.arguments[TypeCtrArg.LIST]))])
+        return self
+
 
 class ConstraintCount(ConstraintWithCondition):
     def __init__(self, lst, values, condition):
@@ -428,7 +470,7 @@ class ConstraintCardinality(Constraint):
 
 
 def _index_att(v):
-    return [(TypeCtrArg.START_INDEX, v)] if v is not None and v != 0 else []
+    return []  # [(TypeCtrArg.START_INDEX, v)] if v is not None and v != 0 else []
 
 
 class ConstraintMaximum(ConstraintWithCondition):
@@ -440,10 +482,10 @@ class ConstraintMaximum(ConstraintWithCondition):
         self.arg(TypeCtrArg.CONDITION, condition)
 
     def min_possible_value(self):
-        return max(x.dom.smallest_value() for x in self.arguments[TypeCtrArg.LIST].content)
+        return max(x.dom.smallest_value() if isinstance(x, Variable) else x.possible_values()[0] for x in self.arguments[TypeCtrArg.LIST].content)
 
     def max_possible_value(self):
-        return max(x.dom.greatest_value() for x in self.arguments[TypeCtrArg.LIST].content)
+        return max(x.dom.greatest_value() if isinstance(x, Variable) else x.possible_values()[-1] for x in self.arguments[TypeCtrArg.LIST].content)
 
 
 class ConstraintMinimum(ConstraintMaximum):
@@ -452,10 +494,10 @@ class ConstraintMinimum(ConstraintMaximum):
         self.name = TypeCtr.MINIMUM
 
     def min_possible_value(self):
-        return min(x.dom.smallest_value() for x in self.arguments[TypeCtrArg.LIST].content)
+        return min(x.dom.smallest_value() if isinstance(x, Variable) else x.possible_values()[0] for x in self.arguments[TypeCtrArg.LIST].content)
 
     def max_possible_value(self):
-        return min(x.dom.greatest_value() for x in self.arguments[TypeCtrArg.LIST].content)
+        return min(x.dom.greatest_value() if isinstance(x, Variable) else x.possible_values()[-1] for x in self.arguments[TypeCtrArg.LIST].content)
 
 
 class ConstraintElement(ConstraintWithCondition):  # currently, not exactly with a general condition
@@ -464,7 +506,9 @@ class ConstraintElement(ConstraintWithCondition):  # currently, not exactly with
         smallest = index.dom[0].smallest() if isinstance(index.dom[0], IntegerEntity) else index.dom[0]
         self.arg(TypeCtrArg.LIST, lst, content_ordered=index is not None, attributes=_index_att(smallest))
         self.arg(TypeCtrArg.INDEX, index, attributes=[(TypeCtrArg.RANK, type_rank)] if type_rank else [])
-        self.arg(TypeCtrArg.VALUE, value)
+        if value:
+            self.arg(TypeCtrArg.CONDITION, Condition.build_condition((TypeConditionOperator.EQ, value)))
+        # self.arg(TypeCtrArg.VALUE, value)
 
     def min_possible_value(self):
         if isinstance(self.arguments[TypeCtrArg.LIST].content[0], int):
@@ -477,14 +521,27 @@ class ConstraintElement(ConstraintWithCondition):  # currently, not exactly with
         return max(x.dom.greatest_value() for x in self.arguments[TypeCtrArg.LIST].content)
 
 
-class ConstraintElementMatrix(Constraint):
+class ConstraintElementMatrix(ConstraintWithCondition):
     def __init__(self, lst, index1, index2, value=None, start_row_index=0, start_col_index=0):
         super().__init__(TypeCtr.ELEMENT)
+        self.matrix = lst
         self.arg(TypeCtrArg.MATRIX, matrix_to_string(lst), content_compressible=lst,
                  attributes=([(TypeCtrArg.ST.START_ROW_INDEX, start_row_index)] if start_row_index else []) + (
                      [(TypeCtrArg.START_COL_INDEX, start_col_index)] if start_col_index else []))
         self.arg(TypeCtrArg.INDEX, [index1, index2], content_ordered=True)
-        self.arg(TypeCtrArg.VALUE, value)
+        if value:
+            self.arg(TypeCtrArg.CONDITION, Condition.build_condition((TypeConditionOperator.EQ, value)))
+        # self.arg(TypeCtrArg.VALUE, value)
+
+    def min_possible_value(self):
+        if isinstance(self.matrix[0][0], int):
+            return min(v for row in self.matrix for v in row)
+        return min(x.dom.smallest_value() for row in self.matrix for x in row)
+
+    def max_possible_value(self):
+        if isinstance(self.matrix[0][0], int):
+            return max(v for row in self.matrix for v in row)
+        return max(x.dom.greatest_value() for row in self.matrix for x in row)
 
 
 class ConstraintChannel(ConstraintUnmergeable):
@@ -517,7 +574,7 @@ class ConstraintNoOverlap(ConstraintUnmergeable):
             self.attributes.append((TypeCtrArg.ZERO_IGNORED, zero_ignored))
 
 
-class ConstraintCumulative(Constraint):
+class ConstraintCumulative(Constraint):  # TODO inheriting from ConstraintWithCondition instead? is this a pb?
     def __init__(self, origins, lengths, ends, heights, condition):
         super().__init__(TypeCtr.CUMULATIVE)
         self.arg(TypeCtrArg.ORIGINS, origins, content_ordered=True)
@@ -527,6 +584,14 @@ class ConstraintCumulative(Constraint):
         self.arg(TypeCtrArg.CONDITION, condition)
 
 
+class ConstraintBinPacking(Constraint):
+    def __init__(self, lst, sizes, condition):
+        super().__init__(TypeCtr.BIN_PACKING)
+        self.arg(TypeCtrArg.LIST, lst, content_ordered=True)
+        self.arg(TypeCtrArg.SIZES, sizes, content_ordered=True)
+        self.arg(TypeCtrArg.CONDITION, condition)
+
+
 ''' Constraints on Graphs'''
 
 
@@ -597,7 +662,8 @@ class PartialConstraint:  # constraint whose condition has not been given such a
             if isinstance(self.constraint, ConstraintElement):
                 arg = self.constraint.arguments[TypeCtrArg.LIST]
                 arg.content = flatten(arg.content)  # we need to flatten now because it has not been done before
-            return ECtr(self.constraint.set_value(other))  # only value must be replaced for these constraints
+            # return ECtr(self.constraint.set_value(other))  # only value must be replaced for these constraints
+            return ECtr(self.constraint.set_condition(TypeConditionOperator.EQ, other))  # the condition must be replaced
         return self.add_condition(TypeConditionOperator.EQ, other)
         # pair = self._simplify_with_auxiliary_variables(other)
         # return Node.build(TypeNode.EQ, pair) if pair else self.add_condition(TypeConditionOperator.EQ, other)
@@ -626,7 +692,11 @@ class PartialConstraint:  # constraint whose condition has not been given such a
         return Node.build(TypeNode.SUB, pair) if pair else PartialConstraint.combine_partial_objects(self, TypeNode.SUB, other)
 
     def __mul__(self, other):
-        assert isinstance(self.constraint, ConstraintSum) and isinstance(other, int)
+        if not isinstance(other, int):
+            pair = self._simplify_operation(other)
+            return Node.build(TypeNode.MUL, pair) if pair else PartialConstraint.combine_partial_objects(self, TypeNode.MUL, other)
+        if not isinstance(self.constraint, ConstraintSum):
+            return Node.build(TypeNode.MUL, self._simplify_operation(other))
         args = self.constraint.arguments
         cs = args[TypeCtrArg.COEFFS].content if TypeCtrArg.COEFFS in args else [1] * len(args[TypeCtrArg.LIST].content)
         value = args[TypeCtrArg.CONDITION]
@@ -638,6 +708,16 @@ class PartialConstraint:  # constraint whose condition has not been given such a
     def __rmul__(self, other):
         return PartialConstraint.__mul__(self, other)
 
+    def __floordiv__(self, other):
+        if isinstance(other, PartialConstraint):
+            other = auxiliary().replace_partial_constraint(other)
+        return Node.build(TypeNode.DIV, auxiliary().replace_partial_constraint(self), other)
+
+    def __mod__(self, other):
+        if isinstance(other, PartialConstraint):
+            other = auxiliary().replace_partial_constraint(other)
+        return Node.build(TypeNode.MOD, auxiliary().replace_partial_constraint(self), other)
+
     def __getitem__(self, i):
         assert isinstance(self.constraint, ConstraintElement)
         lst = self.constraint.arguments[TypeCtrArg.LIST].content
@@ -652,7 +732,11 @@ class PartialConstraint:  # constraint whose condition has not been given such a
     def __str__(self):
         c = self.constraint
         # assert len(c.arguments) == 2
-        return str(c.name) + "(" + str(compact(c.arguments[TypeCtrArg.LIST].content)) + ")"  # TODO experimental stuff
+        s = str(c.name) + "("
+        for k, v in c.arguments.items():
+            s = s + str(v) + ","
+        return s[:-1] + ")"
+        # return s + str(compact(c.arguments[TypeCtrArg.LIST].content)) + ")"
 
     @staticmethod
     def combine_partial_objects(obj1, operator, obj2):  # currently, only partial sums can be combined
@@ -664,7 +748,7 @@ class PartialConstraint:  # constraint whose condition has not been given such a
         pair = obj2.var_val_if_binary_type(TypeNode.MUL) if isinstance(obj2, Node) else None
         if pair:
             obj2 = PartialConstraint(ConstraintSum([pair[0]], [pair[1]], None))
-        elif isinstance(obj2, Variable):
+        elif isinstance(obj2, (Variable, Node)):
             obj2 = PartialConstraint(ConstraintSum([obj2], [1], None))
         elif isinstance(obj2, ScalarProduct):
             obj2 = PartialConstraint(ConstraintSum(obj2.variables, obj2.coeffs, None))
@@ -717,12 +801,25 @@ class ScalarProduct:
     def __sub__(self, other):
         return PartialConstraint.combine_partial_objects(self, TypeNode.SUB, other)
 
+    def __floordiv__(self, other):
+        pc = PartialConstraint(ConstraintSum(self.variables, self.coeffs, None))
+        if isinstance(other, PartialConstraint):
+            other = auxiliary().replace_partial_constraint(other)
+        return Node.build(TypeNode.DIV, auxiliary().replace_partial_constraint(pc), other)
+
+    def __mod__(self, other):
+        pc = PartialConstraint(ConstraintSum(self.variables, self.coeffs, None))
+        if isinstance(other, PartialConstraint):
+            other = auxiliary().replace_partial_constraint(other)
+        return Node.build(TypeNode.MOD, auxiliary().replace_partial_constraint(pc), other)
+
 
 class _Auxiliary:
     def __init__(self):
         self._introduced_variables = []
         self._collected_constraints = []
         self.prefix = "aux_gb"
+        self.cache = []
 
     def __replace(self, obj, dom):
         assert dom.get_type() == TypeVar.INTEGER
@@ -734,26 +831,45 @@ class _Auxiliary:
             self._introduced_variables = EVarArray([var], self.prefix, self.prefix + "[i] is the ith auxiliary variable having been automatically introduced")
         else:
             self._introduced_variables.extend_with(var)
-        self._collected_constraints.append((obj, var))
+        if obj:
+            self._collected_constraints.append((obj, var))
         return var
 
     def replace_partial_constraint(self, pc):
         assert isinstance(pc, PartialConstraint)
-        return self.__replace(pc, Domain(range(pc.constraint.min_possible_value(), pc.constraint.max_possible_value() + 1)))
+        for c, x in self.cache:
+            if pc.constraint.equal_except_condition(c):
+                # if functions.protect().execute(pc.constraint.equal_except_condition(c)):
+                # if functions.protect().execute(pc.constraint == c):
+                return x
+        aux = self.__replace(pc, Domain(range(pc.constraint.min_possible_value(), pc.constraint.max_possible_value() + 1)))
+        self.cache.append((pc.constraint, aux))
+        return aux
 
-    def replace_partial_constraints(self, terms):
+    def replace_nodes_and_partial_constraints(self, terms, nodes_too=False):
         assert isinstance(terms, list)
         for i, t in enumerate(terms):
             if isinstance(t, PartialConstraint):
                 terms[i] = self.replace_partial_constraint(t)
+            elif nodes_too and isinstance(t, Node):
+                terms[i] = self.replace_node(t)
         return terms
 
-    def replace_node(self, node):
+    def replace_node(self, node, indexing=None):
         assert isinstance(node, Node)
-        values = node.possible_values()
-        values = {v for v in values} if len(values) <= 2 else values  # in order to avoid having a range for just 1 or 2 values
+        if indexing:
+            values = indexing  # for the moment, because this is a range for an Element constraint
+        else:
+            values = node.possible_values()
+            values = {v for v in values} if len(values) <= 2 else values  # in order to avoid having a range for just 1 or 2 values
         return self.__replace(node, Domain(values))
 
+    def replace_element_index(self, length, index):
+        if all(0 <= v < length for v in index.dom):
+            return None
+        values = possible_range({v for v in index.dom if 0 <= v < length})
+        return self.__replace(None, Domain(values))
+
     def collected(self):
         t = self._collected_constraints
         self._collected_constraints = []
@@ -764,3 +880,49 @@ def auxiliary():
     if not hasattr(auxiliary, "obj"):
         auxiliary.obj = _Auxiliary()
     return auxiliary.obj
+
+
+def global_indirection(c):
+    pc = None
+    if options.usemeta:
+        return None  # to force using meta-constraints
+    if isinstance(c, ConstraintWithCondition):
+        condition = c.arguments[TypeCtrArg.CONDITION].content
+        c.arguments[TypeCtrArg.CONDITION] = None
+        pc = PartialConstraint(c)
+    if isinstance(c, ConstraintElement):
+        index = c.arguments[TypeCtrArg.INDEX].content
+        length = len(c.arguments[TypeCtrArg.LIST].content)
+        aux = auxiliary().replace_element_index(length, index)
+        if aux:
+            c.arguments[TypeCtrArg.INDEX].content = aux
+            functions.satisfy((index, aux) in {(v, v if 0 <= v < length else ANY) for v in index.dom})
+    if isinstance(c, ConstraintAllDifferent):
+        lst = c.arguments[TypeCtrArg.LIST].content
+        pc = PartialConstraint(ConstraintNValues(lst, c.arguments[TypeCtrArg.EXCEPT].content, None))
+        condition = Condition.build_condition((TypeConditionOperator.EQ, len(lst)))
+    if isinstance(c, ConstraintAllEqual):
+        pc = PartialConstraint(ConstraintNValues(c.arguments[TypeCtrArg.LIST].content, None, None))
+        condition = Condition.build_condition((TypeConditionOperator.EQ, 1))
+    if pc is None:
+        warning("You use a global constraint in a complex expression.\n" +
+                "\tHowever, this constraint cannot be externalized by introducing an auxiliary variable.\n" +
+                "\tHence a meta-constraint function (Or, And, Not, IfThen, IfThenElse, Iff) is automatically called.\n" +
+                "\tCheck that it is relevant, or modify your model.\n")
+        return None
+    return Node.build(condition.operator, auxiliary().replace_partial_constraint(pc), condition.right_operand())
+
+
+def manage_global_indirection(*args):
+    assert len(args) > 1
+    if any(isinstance(arg, EMetaCtr) for arg in args):
+        return None
+    t = []
+    for arg in args:
+        if isinstance(arg, ECtr):
+            gi = global_indirection(arg.constraint)
+            if gi is None:
+                return None
+            arg = gi
+        t.append(arg)
+    return tuple(t)
