@@ -17,12 +17,12 @@ from pycsp3.tools.aggregator import build_similar_constraints
 from pycsp3.tools.compactor import build_compact_forms
 from pycsp3.tools.curser import OpOverrider, convert_to_namedtuples, is_namedtuple
 from pycsp3.tools.slider import handle_slides
-from pycsp3.tools.utilities import Stopwatch, GREEN, WHITE
+from pycsp3.tools.utilities import Stopwatch, GREEN, WHITE, Error
 from pycsp3.tools.xcsp import build_document
 
 None_Values = ['None', '', 'null']  # adding 'none'?
 
-ABSCON, CHOCO = SOLVERS = ["AbsCon", "Choco"]
+ACE, CHOCO = SOLVERS = ["Ace", "Choco"]
 
 
 class Compilation:
@@ -34,20 +34,25 @@ class Compilation:
     stopwatch = None
     stopwatch2 = None
     done = False
+    user_filename = None
 
     @staticmethod
     def load(console=False):
         _load(console=console)
 
     @staticmethod
-    def compile():
-        return _compile()
+    def set_filename(_user_filename):
+        Compilation.user_filename = _user_filename
+
+    @staticmethod
+    def compile(disabling_opoverrider=True):
+        return _compile(disabling_opoverrider)
 
 
 def _load_options():
     options.set_values("data", "dataparser", "dataexport", "dataformat", "variant", "checker", "solver")
     options.set_flags("dataexport", "compress", "ev", "display", "time", "noComments", "recognizeSlides", "keepSmartConditions", "restrictTablesWrtDomains",
-                      "solve")
+                      "safe", "solve", "dontcompactValues", "usemeta", "debug")
     if options.checker is None:
         options.checker = "fast"
     assert options.checker in {"complete", "fast", "none"}
@@ -62,7 +67,7 @@ def _load_model():
         specification = util.spec_from_file_location("", name)
         model = util.module_from_spec(specification)
         return model, model_string
-    except:
+    except Exception:
         usage("It was not possible to read the file: " + sys.argv[0])
         raise
 
@@ -73,57 +78,76 @@ def _load_data():
         return OrderedDict([("f" + str(i), od[i]) for i, v in enumerate(raw_data)]), od
         # return DataVisitor(raw_data).visit(ast.parse(inspect.getsource(Compilation.model)))
 
+    def _arg_value(s):
+        return None if s in None_Values else int(s) if s.isdigit() else s
+
+    def _load_multiple_data_pieces():  # formatting instructions not possible in that case
+        s = ""
+        for arg in args:
+            if "=" in arg:
+                t = arg.split('=')
+                value = _arg_value(t[1])
+                compilation_data[t[0]] = value
+                s += "-" + str(value)
+            else:
+                assert arg.endswith("json")
+                assert os.path.exists(arg), "The file " + arg + " does not exist (in the specified directory)." + str(os.path)
+                with open(arg) as f:
+                    compilation_data.update(json.loads(f.read(), object_pairs_hook=OrderedDict))
+                    s += "-" + arg.split(os.sep)[-1:][0].split(".")[:1][0]
+        return compilation_data, s
+
     data = options.data
-    compilation_data = OrderedDict()  # the object used for recording the data, available in the model
     if data is None:
-        return compilation_data, ""
-    if data.endswith(".json"):
+        return OrderedDict(), ""
+    if data.endswith(".json"):  # a single json file
         assert os.path.exists(data), "The file " + data + " does not exist (in the specified directory)."
         with open(data) as f:
-            compilation_data = json.loads(f.read(), object_pairs_hook=OrderedDict)
-            string_data = "-" + data.split(os.sep)[-1:][0].split(".")[:1][0]
+            return json.loads(f.read(), object_pairs_hook=OrderedDict), "-" + data.split(os.sep)[-1:][0].split(".")[:1][0]
+    compilation_data = OrderedDict()  # the object used for recording the data, available in the model
+    # if '{' in data and '}' in data:
+    #    compilation_data = json.loads(data, object_hook=lambda d: namedtuple('X', d.keys())(*d.values()), object_pairs_hook=OrderedDict)
+    #    for k, v in compilation_data.items(): setattr(compilation_data, k, v)  ordered_data = list(compilation_data.values())
+    if (data[0], data[-1]) in [('[', ']'), ('(', ')')]:  # NB: these characters may be needed to be escaped as in \[2,3\]
+        args = data[1:-1].split(",")
+        if "json" in data:
+            return _load_multiple_data_pieces()
+        if '=' in data:
+            assert data.count('=') == data.count(',') + 1, "badly formed string of data " + data
+            ordered_data = []
+            for arg in args:
+                t = arg.split('=')
+                value = _arg_value(t[1])
+                compilation_data[t[0]] = value
+                ordered_data.append(value)
+        else:
+            compilation_data, ordered_data = _load_data_sequence(args)
     else:
-        #  if '{' in data and '}' in data:
-        #    compilation_data = json.loads(data, object_hook=lambda d: namedtuple('X', d.keys())(*d.values()), object_pairs_hook=OrderedDict)
-        #    for k, v in compilation_data.items(): setattr(compilation_data, k, v)  ordered_data = list(compilation_data.values())
-        if data[0] == '[' and data[-1] == ']' or data[0] == '(' and data[-1] == ')':  # NB: these characters may be needed to be escaped as in \[2,3\]
-            args = data[1:-1].split(",")
-            if '=' in data:
-                assert data.count('=') == data.count(',') + 1, "badly formed string of data " + data
-                ordered_data = []
-                for arg in args:
-                    t = arg.split('=')
-                    value = None if t[1] in None_Values else int(t[1]) if t[1].isdigit() else t[1]
-                    compilation_data[t[0]] = value
-                    ordered_data.append(value)
-            else:
-                compilation_data, ordered_data = _load_data_sequence(args)
-        else:
-            compilation_data, ordered_data = _load_data_sequence([data])
-        df = options.dataformat
-        if df:
-            if df[0] == '[':
-                assert df[-1] == ']'
-                df = df[1:-1]
-            df = df.split(',')
-            assert len(df) == len(ordered_data)
-            ss = "-".join(df).format(*ordered_data)
-        else:
-            ss = "-".join(str(v) for v in ordered_data)
-        string_data = "-" + ss
+        compilation_data, ordered_data = _load_data_sequence([data])
+    df = options.dataformat
+    if df:
+        if df[0] == '[':
+            assert df[-1] == ']'
+            df = df[1:-1]
+        df = df.split(',')
+        assert len(df) == len(ordered_data)
+        ss = "-".join(df).format(*ordered_data)
+    else:
+        ss = "-".join(str(v) for v in ordered_data)
+    string_data = "-" + ss
     return compilation_data, string_data
 
 
-def _load_dataparser(parser_file, data_file):
+def _load_dataparser(parser_file, data_value):
     try:
-        compilation_data = parsing.register_fields(data_file)  # the object used for recording data is returned, available in the model
+        compilation_data = parsing.register_fields(data_value)  # the object used for recording data is returned, available in the model
         specification = util.spec_from_file_location("", parser_file)
         specification.loader.exec_module(util.module_from_spec(specification))
         string_data = "-" + options.data.split(os.sep)[-1:][0].split(".")[:1][0] if options.data else None
         if string_data is None:
             string_data = Compilation.string_data if Compilation.string_data else ""  # in case data are recorded through the dataparser (after asking the user)
         return compilation_data, string_data
-    except:
+    except Exception:
         usage("It was not possible to correctly read the file: " + parser_file)
         raise
 
@@ -138,6 +162,7 @@ def _load(*, console=False):
         else:
             Compilation.data, Compilation.string_data = _load_data()
         Compilation.data = convert_to_namedtuples(Compilation.data)
+        Compilation.string_data = Compilation.string_data.replace("/", "-")
         if len(Compilation.data) == 0:
             Compilation.data = None
         elif len(Compilation.data) == 1:
@@ -161,7 +186,7 @@ def default_data(filename):
     return Compilation.data
 
 
-def _compile():
+def _compile(disabling_opoverrider=False):
     # used to save data in jSON
     def prepare_for_json(obj):
         if is_namedtuple(obj):
@@ -175,16 +200,18 @@ def _compile():
             return obj
         return str(obj) if isinstance(obj, datetime.time) else obj
 
-    OpOverrider.disable()
-    # if options.display:
-    #     if sys.argv[1].endswith(".json"):
-    #         with open(sys.argv[1], 'r') as f:
-    #             print(f.read())
+    if Error.errorOccurrence:
+        return None
 
-    filename_prefix = Compilation.string_model + ("-" + options.variant if options.variant else "") + Compilation.string_data
-    filename = filename_prefix + ".xml"
-    if Compilation.done:
-        return filename
+    if disabling_opoverrider:
+        OpOverrider.disable()
+
+    if Compilation.user_filename is not None:
+        print("  * User-defined XML file name:", Compilation.user_filename)
+        filename = Compilation.user_filename
+    else:
+        filename_prefix = Compilation.string_model + ("-" + options.variant if options.variant else "") + Compilation.string_data
+        filename = filename_prefix + ".xml"
 
     stopwatch = Stopwatch()
     print("  PyCSP3 (Python:" + platform.python_version() + ", Path:" + os.path.abspath(__file__) + ")\n")
@@ -224,7 +251,8 @@ def _compile():
 
     Compilation.done = True
 
-    solving = ABSCON if options.solve else options.solver
+    cop = root is not None and root.attrib and root.attrib["type"] == "COP"
+    solving = ACE if options.solve else options.solver
     if solving:
         if options.display:
             print("Warning: options -display and -solve should not be used together.")
@@ -232,19 +260,18 @@ def _compile():
         solver, args, args_recursive = process_options(solving)
         solver = next(ss for ss in SOLVERS if ss.lower() == solver.lower())
         # print("solver", solver, "args", args)
-        cop = root.attrib["type"] == "COP"
         if solver == CHOCO:
             from pycsp3.solvers.choco import ChocoProcess
-            result, solution = ChocoProcess().solve(filename, solving, args, args_recursive, cop)
-        else:  # Fallback case => options.solver == "abscon":
-            from pycsp3.solvers.abscon import AbsConProcess
-            result, solution = AbsConProcess().solve(filename, solving, args, args_recursive, cop)
+            result, solution = ChocoProcess().solve((filename, cop), solving, args, args_recursive, compiler=True, automatic=True)
+        else:  # Fallback case => options.solver == "ace":
+            from pycsp3.solvers.abscon import AceProcess
+            result, solution = AceProcess().solve((filename, cop), solving, args, args_recursive, compiler=True, automatic=True)
         # if result:
         #     print(result)
         if solution:
             print(solution)
 
-    return filename
+    return filename, cop
 
 
 def usage(message):
