@@ -8,7 +8,9 @@ from pycsp3.classes.auxiliary.conditions import Condition, ConditionInterval, Co
 from pycsp3.classes.auxiliary.ptypes import TypeOrderedOperator, TypeConditionOperator, TypeVar, TypeCtr, TypeCtrArg, TypeRank
 from pycsp3.classes.auxiliary.structures import Automaton, MDD
 from pycsp3.classes.entities import (
-    EVar, EVarArray, ECtr, ECtrs, EToGather, EToSatisfy, EBlock, ESlide, EIfThenElse, EObjective, EAnnotation, AnnEntities, TypeNode, Node)
+    EVar, EVarArray, ECtr, EMetaCtr, ECtrs, EToGather, EToSatisfy, EBlock, ESlide, EAnd, EOr, ENot, EXor, EIfThen, EIfThenElse, EIff, EObjective, EAnnotation,
+    AnnEntities,
+    TypeNode, Node)
 from pycsp3.classes.main.annotations import (
     AnnotationDecision, AnnotationOutput, AnnotationVarHeuristic, AnnotationValHeuristic, AnnotationFiltering, AnnotationPrepro, AnnotationSearch,
     AnnotationRestarts)
@@ -16,10 +18,11 @@ from pycsp3.classes.main.constraints import (
     ConstraintIntension, ConstraintExtension, ConstraintRegular, ConstraintMdd, ConstraintAllDifferent,
     ConstraintAllDifferentList, ConstraintAllDifferentMatrix, ConstraintAllEqual, ConstraintOrdered, ConstraintLex, ConstraintLexMatrix, ConstraintSum,
     ConstraintCount, ConstraintNValues, ConstraintCardinality, ConstraintMaximum, ConstraintMinimum, ConstraintChannel, ConstraintNoOverlap,
-    ConstraintCumulative, ConstraintCircuit, ConstraintClause, PartialConstraint, ScalarProduct, auxiliary)
+    ConstraintCumulative, ConstraintBinPacking, ConstraintCircuit, ConstraintClause, PartialConstraint, ScalarProduct, auxiliary, global_indirection,
+    manage_global_indirection)
 from pycsp3.classes.main.domains import Domain
 from pycsp3.classes.main.objectives import ObjectiveExpression, ObjectivePartial
-from pycsp3.classes.main.variables import Variable, VariableInteger, VariableSymbolic, NotVariable, NegVariable
+from pycsp3.classes.main.variables import Variable, VariableInteger, VariableSymbolic
 from pycsp3.compiler import default_data
 from pycsp3.dashboard import options
 from pycsp3.tools import curser
@@ -90,7 +93,7 @@ def VarArray(*, size, dom, comment=None):
     size = [size] if isinstance(size, int) else size
     assert all(dimension != 0 for dimension in size), "No dimension must not be equal to 0"
     checkType(size, [int])
-    checkType(dom, (range, Domain, [int, range, str, Domain, type(None)], type(lambda: 0)))
+    # checkType(dom, (range, Domain, [int, range, str, Domain, type(None)], type(lambda: 0)))  # TODO a problem with large sets
     name = extract_declaration_for("VarArray")
     if comment is None and not isinstance(name, list):
         comment, tags = comment_and_tags_of(function_name="VarArray")
@@ -103,6 +106,11 @@ def VarArray(*, size, dom, comment=None):
     assert isinstance(comment, (str, type(None))), "A comment must be a string (or None). Usually, they are given on plain lines preceding the declaration"
     assert str(name) not in Variable.name2obj, "The identifier " + name + " is used twice. This is not possible"
 
+    if isinstance(dom, (tuple, list, set)):
+        domain = flatten(dom)
+        assert all(isinstance(v, int) for v in domain) or all(isinstance(v, str) for v in domain)
+        dom = Domain(set(domain))
+
     var_objects = Variable.build_variables_array(name, size, dom)
     if isinstance(name, list):
         for variable in var_objects:
@@ -121,25 +129,33 @@ def _bool_interpretation_for_in(left_operand, right_operand, bool_value):
 
     if isinstance(left_operand, Variable) and isinstance(right_operand, (tuple, list, set, frozenset, range)) and len(right_operand) == 0:
         return None
-    if isinstance(left_operand, (Variable, int, str)) and isinstance(right_operand, (set, frozenset, range)):
+    if isinstance(left_operand, Variable) and isinstance(right_operand, range):
+        ctr = Extension(scope=[left_operand], table=list(right_operand), positive=bool_value)
+    elif isinstance(left_operand, (Variable, int, str)) and isinstance(right_operand, (set, frozenset, range)):
         # it is a unary constraint of the form x in/not in set/range
         ctr = Intension(Node.build(TypeNode.IN, left_operand, right_operand) if bool_value else Node.build(TypeNode.NOTIN, left_operand, right_operand))
+    # elif isinstance(left_operand, Node) and isinstance(right_operand, range):
+    #     if bool_value:
+    #         ctr = Intension(conjunction(left_operand >= right_operand.start, left_operand <= right_operand.stop - 1))
+    #     else:
+    #         ctr = Intension(disjunction(left_operand < right_operand.start, left_operand > right_operand.stop - 1))
     elif isinstance(left_operand, PartialConstraint):  # it is a partial form of constraint (sum, count, maximum, ...)
         ctr = ECtr(left_operand.constraint.set_condition(TypeConditionOperator.IN if bool_value else TypeConditionOperator.NOTIN, right_operand))
-    elif isinstance(right_operand, Automaton):  #  it is a regular constraint
+    elif isinstance(right_operand, Automaton):  # it is a regular constraint
         ctr = Regular(scope=left_operand, automaton=right_operand)
     elif isinstance(right_operand, MDD):  # it is a MDD constraint
         ctr = Mdd(scope=left_operand, mdd=right_operand)
     elif isinstance(left_operand, int) and (is_1d_list(right_operand, Variable) or is_1d_tuple(right_operand, Variable)):
         ctr = Count(right_operand, value=left_operand, condition=(TypeConditionOperator.GE, 1))  # atLeast1 TODO to be replaced by a member/element constraint ?
-    else:  #  It is a table constraint
-
+    # elif isinstance(left_operand, Node):
+    #
+    else:  # It is a table constraint
         if not hasattr(left_operand, '__iter__'):
             left_operand = [left_operand]
         if not bool_value and len(right_operand) == 0:
             return None
         # TODO what to do if the table is empty and bool_value is true? an error message ?
-        ctr = Extension(scope=list(left_operand), table=right_operand, positive=bool_value)
+        ctr = Extension(scope=flatten(left_operand), table=right_operand, positive=bool_value)  # TODO ok for using flatten? (before it was list())
     return ctr
 
 
@@ -162,10 +178,34 @@ def _wrap_intension_constraints(entities):
     return entities
 
 
+def And(*args):
+    return EAnd(_wrap_intension_constraints(_complete_partial_forms_of_constraints(flatten(*args))))
+
+
+def Or(*args):
+    return EOr(_wrap_intension_constraints(_complete_partial_forms_of_constraints(flatten(*args))))
+
+
+def Not(*args):
+    return ENot(_wrap_intension_constraints(_complete_partial_forms_of_constraints(flatten(*args))))
+
+
+def Xor(*args):
+    return EXor(_wrap_intension_constraints(_complete_partial_forms_of_constraints(flatten(*args))))
+
+
+def IfThen(*args):
+    return EIfThen(_wrap_intension_constraints(_complete_partial_forms_of_constraints(flatten(*args))))
+
+
 def IfThenElse(*args):
     return EIfThenElse(_wrap_intension_constraints(_complete_partial_forms_of_constraints(flatten(*args))))
 
 
+def Iff(*args):
+    return EIff(_wrap_intension_constraints(_complete_partial_forms_of_constraints(flatten(*args))))
+
+
 def Slide(*args):
     entities = _wrap_intension_constraints(
         flatten(*args))  # we cannot directly complete partial forms (because it is executed before the analysis of the parameters of satisfy
@@ -178,7 +218,7 @@ def satisfy(*args):
 
     def _group(*_args):
         entities = _wrap_intension_constraints(_complete_partial_forms_of_constraints(flatten(*_args)))
-        checkType(entities, [ECtr, ECtrs])
+        checkType(entities, [ECtr, ECtrs, EMetaCtr])
         return EToGather(entities)
 
     def _block(*_args):
@@ -228,13 +268,15 @@ def satisfy(*args):
                     if isinstance(l, list) and len(l) > 0 and isinstance(l[0], tuple):
                         arg[j] = _reorder(l)
         no_parameter_satisfy = i
-        assert isinstance(arg, (ECtr, ESlide, Node, bool, list, tuple, type(None), types.GeneratorType)), "non authorized type " + str(arg) + " " + str(
-            type(arg))
+        if isinstance(arg, (set,frozenset)):
+            arg= list(arg)
+        assert isinstance(arg, (ECtr, EMetaCtr, ESlide, Node, bool, list, tuple, type(None), types.GeneratorType)), \
+            "non authorized type " + str(arg) + " " + str(type(arg))
         if arg is None:
             continue
         arg = list(arg) if isinstance(arg, types.GeneratorType) else arg
         comment_at_2 = any(comment != '' for comment in comments2[i])
-        if isinstance(arg, (ECtr, ESlide)):  # case: simple constraint or slide
+        if isinstance(arg, (ECtr, EMetaCtr, ESlide)):  # case: simple constraint or slide
             to_post = _complete_partial_forms_of_constraints([arg])[0]
         elif isinstance(arg, Node):  # a predicate to be wrapped by a constraint (intension)
             to_post = Intension(arg)
@@ -244,7 +286,7 @@ def satisfy(*args):
             #                 + str(args) + " " + str(i) + ".\nA constraint is certainly badly formed"
             other, partial = curser.queue_in.popleft()
             to_post = _bool_interpretation_for_in(partial, other, arg)
-        elif any(isinstance(ele, ESlide) for ele in arg):  #  Case: Slide
+        elif any(isinstance(ele, ESlide) for ele in arg):  # Case: Slide
             to_post = _block(arg)
         elif comment_at_2:  # Case: block
             for j, ele in enumerate(arg):
@@ -253,7 +295,7 @@ def satisfy(*args):
                 elif comments2[i][j] or tags2[i][j]:
                     arg[j] = _group(arg[j]).note(comments2[i][j]).tag(tags2[i][j])
             to_post = _block(arg)
-        else:  #  Group
+        else:  # Group
             to_post = _group(arg)
         if to_post:
             t.append(to_post.note(comments1[i]).tag(tags1[i]))
@@ -277,7 +319,7 @@ def Extension(*, scope, table, positive=True):
         assert all(isinstance(v, int) if isinstance(scope[0], VariableInteger) else isinstance(v, str) for v in table)
     else:  # if all(isinstance(x, VariableInteger) for x in scope):
         for i, t in enumerate(table):
-            assert isinstance(t, tuple)
+            assert isinstance(t, tuple), str(t)
             assert len(t) == len(scope), ("The length of each tuple must be the same as the arity."
                                           + "Maybe a problem with slicing: you must for example write x#[i:i+3,0] instead of x[i:i+3][0]")
     return ECtr(ConstraintExtension(scope, table, positive, options.keepsmartconditions, options.restricttableswrtdomains))
@@ -306,24 +348,33 @@ def max(*args):
     return Node.build(TypeNode.MAX, *args) if len(args) > 0 and any(isinstance(a, (Node, Variable)) for a in args) else maxPython(*args)
 
 
-def dist(*args):
-    return Node.build(TypeNode.DIST, *args)
-
-
 def xor(*args):
-    return Node.build(TypeNode.XOR, *args) if len(args) > 1 else args[0]
+    return args[0] ^ args[1] if len(args) == 2 else Node.build(TypeNode.XOR, *args) if len(args) > 1 else args[0]
 
 
 def iff(*args):
-    return Node.build(TypeNode.IFF, *args)
+    assert len(args) >= 2
+    res = manage_global_indirection(*args)
+    if res is None:
+        return Iff(args)
+    args = res
+    return args[0] == args[1] if len(args) == 2 else Node.build(TypeNode.IFF, *args)
 
 
 def imply(*args):
-    return Node.build(TypeNode.IMP, *args)
+    assert len(args) == 2
+    res = manage_global_indirection(*args)
+    if res is None:
+        return IfThen(args)
+    return Node.build(TypeNode.IMP, *res)
 
 
 def ift(*args):
-    return Node.build(TypeNode.IF, *args)
+    assert len(args) == 3
+    res = manage_global_indirection(*args)
+    if res is None:
+        return IfThenElse(args)
+    return Node.build(TypeNode.IF, *res)
 
 
 def expr(operator, *args):
@@ -338,11 +389,6 @@ def disjunction(*args):
     return Node.disjunction(*args)
 
 
-def knight_attack(x, y, order):
-    d1, d2 = dist(x // order, y // order), dist(x % order, y % order)  # distances between rows and columns
-    return (d1 == 1) & (d2 == 2) | (d1 == 2) & (d2 == 1)
-
-
 ''' Language-based Constraints '''
 
 
@@ -357,7 +403,7 @@ def Mdd(*, scope, mdd):
     scope = flatten(scope)
     checkType(scope, [Variable])
     checkType(mdd, MDD)
-    return ECtr(ConstraintMdd(scope, mdd.transitions))
+    return ECtr(ConstraintMdd(scope, mdd))
 
 
 ''' Comparison-based Constraints '''
@@ -461,10 +507,10 @@ def Sum(term, *others, condition=None):
         for tree in terms:
             if isinstance(tree, Variable):
                 t1.append(tree)
-                t2.append(1)
+                t2.append(-1 if tree.inverse else 1)
             else:
-                assert isinstance(tree, (Node, NegVariable))
-                pair = (tree.variable, -1) if isinstance(tree, NegVariable) else tree.tree_val_if_binary_type(TypeNode.MUL)
+                assert isinstance(tree, Node)
+                pair = tree.tree_val_if_binary_type(TypeNode.MUL)
                 if pair is None:
                     break
                 t1.append(pair[0])
@@ -481,7 +527,7 @@ def Sum(term, *others, condition=None):
             OpOverrider.disable()
             if len(coeffs) == 1 and isinstance(coeffs[0], (tuple, set, range)):
                 coeffs = list(coeffs[0])
-            elif isinstance(coeffs, (tuple, set, range)):
+            elif isinstance(coeffs, (tuple, set, range)):  # TODO is set appropriate?
                 coeffs = list(coeffs)
             elif isinstance(coeffs, (int, Variable)):
                 coeffs = [coeffs]
@@ -495,16 +541,14 @@ def Sum(term, *others, condition=None):
         return terms, coeffs
 
     terms = list(term) if isinstance(term, types.GeneratorType) else flatten(term, others)
-
     checkType(terms, ([Variable], [Node], [PartialConstraint], [ScalarProduct]))
-    auxiliary().replace_partial_constraints(terms)
+    auxiliary().replace_nodes_and_partial_constraints(terms)
 
     terms, coeffs = _get_terms_coeffs(terms)
     terms, coeffs = _manage_coeffs(terms, coeffs)
     # if len(terms) == 1 and coeffs is None: return terms
 
     # TODO control here some assumptions (empty list seems to be possible. See RLFAP)
-
     return _wrapping_by_complete_or_partial_constraint(ConstraintSum(terms, coeffs, Condition.build_condition(condition)))
 
 
@@ -520,16 +564,6 @@ def Count(term, *others, value=None, values=None, condition=None):
     return _wrapping_by_complete_or_partial_constraint(ConstraintCount(terms, values, Condition.build_condition(condition)))
 
 
-# def AtLeastOne(term, *others, value=None, values=None):
-#     return Count(term, others, value=value, values=values) >= 1
-#
-# def AtMostOne(term, *others, value=None, values=None):
-#     return Count(term, others, value=value, values=values) <= 1
-#
-# def ExactlyOne(term, *others, value=None, values=None):
-#     return Count(term, others, value=value, values=values) == 1
-
-
 def NValues(term, *others, excepting=None, condition=None):
     terms = flatten(term, others)
     checkType(terms, [Variable])
@@ -553,10 +587,12 @@ def Cardinality(term, *others, occurrences, closed=False):
     checkType(closed, (bool, type(None)))
     for i, occ in enumerate(occurs):
         if isinstance(occ, range):
-            occurs[i] = str(min(occ)) + ".." + str(max(occ))
+            occurs[i] = min(occ) if len(occ) == 1 else str(min(occ)) + ".." + str(max(occ))
         if isinstance(occ, list):
             flat = flatten(occ)
-            if all([isinstance(e, int) for e in flat]) and flat == list(range(min(flat), max(flat) + 1)):
+            if len(flat) == 1:
+                flat = flat[0]
+            elif all(isinstance(e, int) for e in flat) and flat == list(range(min(flat), max(flat) + 1)):
                 flat = str(min(flat)) + ".." + str(max(flat))
             occurs[i] = flat
     return ECtr(ConstraintCardinality(terms, values, occurs, closed))
@@ -569,7 +605,7 @@ def _extremum(term, others, index, start_index, type_rank, condition, maximum):
     terms = list(term) if isinstance(term, types.GeneratorType) else flatten(term, others)
     terms = [Sum(t) if isinstance(t, ScalarProduct) else t for t in terms]  # to have PartialConstraints
     checkType(terms, ([Variable, Node], [PartialConstraint]))
-    auxiliary().replace_partial_constraints(terms)
+    auxiliary().replace_nodes_and_partial_constraints(terms)
     checkType(index, (Variable, type(None)))
     checkType(start_index, int)
     checkType(type_rank, TypeRank)
@@ -610,24 +646,51 @@ def Channel(list1, list2=None, *, start_index1=0, start_index2=0):
 ''' Packing and Scheduling Constraints '''
 
 
-def NoOverlap(*, origins, lengths, zero_ignored=False):
+def NoOverlap(tasks=None, *, origins=None, lengths=None, zero_ignored=False):
+    if tasks:
+        assert origins is None and lengths is None
+        tasks = list(tasks) if isinstance(tasks, (tuple, set, frozenset, types.GeneratorType)) else tasks
+        assert isinstance(tasks, list) and len(tasks) > 0
+        assert any(isinstance(task, (tuple, list)) and len(task) == 2 for task in tasks)
+        origins, lengths = zip(*tasks)
     checkType(origins, [Variable])
+    lengths = [lengths for _ in range(len(origins))] if isinstance(lengths, int) else lengths
     checkType(lengths, ([Variable], [int]))
     return ECtr(ConstraintNoOverlap(origins, lengths, zero_ignored))
 
 
-def Cumulative(*, origins, lengths, ends=None, heights, condition=None):
+def Cumulative(tasks=None, *, origins=None, lengths=None, ends=None, heights=None, condition=None):
+    if tasks:
+        assert origins is None and lengths is None and ends is None and heights is None
+        tasks = list(tasks) if isinstance(tasks, (tuple, set, frozenset, types.GeneratorType)) else tasks
+        assert isinstance(tasks, list) and len(tasks) > 0
+        v = len(tasks[0])
+        assert v in (3, 4) and any(isinstance(task, (tuple, list)) and len(task) == v for task in tasks)
+        if v == 3:
+            origins, lengths, heights = zip(*tasks)
+        else:
+            origins, lengths, ends, heights = zip(*tasks)
     origins = flatten(origins)
     checkType(origins, [Variable])
     lengths = flatten(lengths)
     checkType(lengths, ([Variable], [int]))
-    heights = flatten(heights)
+    heights = [heights for _ in range(len(origins))] if isinstance(heights, int) else flatten(heights)
     checkType(heights, ([Variable], [int]))
-    if ends is not None: ends = flatten(ends)
+    ends = flatten(ends) if ends is not None else ends  # ends is optional
     checkType(ends, ([Variable], type(None)))
     return _wrapping_by_complete_or_partial_constraint(ConstraintCumulative(origins, lengths, ends, heights, Condition.build_condition(condition)))
 
 
+def BinPacking(term, *others, sizes, condition=None):
+    terms = flatten(term, others)
+    assert len(terms) > 0, "A binPacking with an empty scope"
+    checkType(terms, [Variable])
+    sizes = flatten(sizes)
+    checkType(sizes, [int])
+    assert len(terms) == len(sizes)
+    return _wrapping_by_complete_or_partial_constraint(ConstraintBinPacking(terms, sizes, Condition.build_condition(condition)))
+
+
 ''' Constraints on Graphs'''
 
 
@@ -646,11 +709,10 @@ def Clause(term, *others, phases=None):
     literals = flatten(term, others)
     phases = [False] * len(literals) if phases is None else flatten(phases)
     assert len(literals) == len(phases)
-    checkType(literals, ([Variable, NotVariable]))
+    checkType(literals, [Variable])
     checkType(phases, [bool])
     for i, literal in enumerate(literals):
-        if isinstance(literal, NotVariable):
-            literals[i] = literal.variable
+        if literal.negation:
             phases[i] = True
     return ECtr(ConstraintClause(literals, phases))
 
@@ -659,10 +721,10 @@ def Clause(term, *others, phases=None):
 
 
 def _optimize(term, minimization):
-    if isinstance(term, PartialConstraint) and isinstance(term.constraint, ConstraintSum) and TypeCtrArg.COEFFS not in term.constraint.arguments:
+    if isinstance(term, PartialConstraint) and isinstance(term.constraint, (ConstraintSum, ConstraintMaximum, ConstraintMinimum)):
         l = term.constraint.arguments[TypeCtrArg.LIST]
-        if len(l.content) == 1:
-            term = l.content[0]  # this was a sum with only one term, so we just consider this term as an expression to be optimized
+        if len(l.content) == 1 and TypeCtrArg.COEFFS not in term.constraint.arguments:
+            term = l.content[0]  # this was a sum/maximum/minimum with only one term, so we just consider this term as an expression to be optimized
     if isinstance(term, ScalarProduct):
         term = Sum(term)  # to have a PartialConstraint
     checkType(term, (Variable, Node, PartialConstraint)), "Did you forget to use Sum, e.g., as in Sum(x[i]*3 for i in range(10))"
