@@ -30,8 +30,11 @@ satisfy(
     AllDifferent(k),
 
     # all knights form a cycle
-    [knight_attack(k[i], k[(i + 1) % nKnights], n) for i in range(nKnights)]
+    [(abs(k[i] // n - k[(i + 1) % nKnights] // n), abs(k[i] % n - k[(i + 1) % nKnights] % n)) in {(1, 2), (2, 1)} for i in range(nKnights)]
 )
 
-# Note that adding  (q[i] != k[j] % n) | (i != k[j] // n) for i in range(n) for j in range(nKnights)
-# does not seem to filter more values.
+""" Comments
+1) adding  (q[i] != k[j] % n) | (i != k[j] // n) for i in range(n) for j in range(nKnights) does not seem to filter more values.
+
+2) expressing a table constraint where the scope does not list simple variables entails automatically introducing auxiliary variables at compilation time
+"""
\ No newline at end of file
