@@ -4,14 +4,14 @@ import subprocess
 import sys
 from pathlib import Path
 
-from pycsp3.solvers.abscon import AbsConProcess
-from pycsp3.tools.utilities import BLUE, GREEN, ORANGE, RED, WHITE, WHITE_BOLD
+from pycsp3.solvers.abscon import AceProcess
+from pycsp3.tools.utilities import is_windows, BLUE, GREEN, ORANGE, RED, WHITE, WHITE_BOLD, warning
 
 COLOR_PY, COLOR_JV = BLUE, ORANGE
 
 PYTHON_VERSIONS = ["python3"]
 waiting = False
-solver = AbsConProcess()
+solver = AceProcess()
 
 
 def run(xcsp, diff=None, same=None):
@@ -32,7 +32,7 @@ def run(xcsp, diff=None, same=None):
     # Get versions
     for i, python_exec in enumerate(PYTHON_VERSIONS):
         cmd = [python_exec, "--version"]
-        out, _ = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()
+        out, _ = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=is_windows()).communicate()
         version = out.decode('utf-8').strip()
         PYTHON_VERSIONS[i] = (python_exec, version)
 
@@ -50,20 +50,20 @@ class Tester:
     def system_command(command, origin, target):
         assert command in {"mv", "cp"}
         print(os.getcwd())
-        if os.name == 'nt':
+        if is_windows():
             command = "move" if command == 'mv' else command
             command = "copy" if command == 'cp' else command
         print(command, origin, target)
         if not os.path.isfile(origin):
-            print("error: do not found the file " + origin)
+            warning("file not found: " + origin)
             exit(0)
-        subprocess.call([command, origin, target], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        subprocess.call([command, origin, target], stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=is_windows())
 
     @staticmethod
     def xml_indent(file):
         cmd = ["pycsp3" + os.sep + "libs" + os.sep + "xmlindent" + os.sep + "xmlindent", '-i', '2', '-w', file]
         print(cmd)
-        out, error = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()
+        out, error = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=is_windows()).communicate()
         if error.decode('utf-8') != "":
             print("XmlIndent stderr : ")
 
@@ -72,10 +72,10 @@ class Tester:
         if len(sys.argv) == 2:
             if sys.argv[1] == "-choco":
                 command += " -solver=[choco,limit=2s]"
-            elif sys.argv[1] == "-abscon":
-                command += " -solver=[abscon,limit=2s]"
+            elif sys.argv[1] == "-ace":
+                command += " -solver=[ace,limit=2s]"
         print(BLUE + "Command:" + WHITE, command)
-        out, error = subprocess.Popen(command.split(), stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()
+        out, error = subprocess.Popen(command.split(), stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=is_windows()).communicate()
         # print(title + " stdout:")
         print(out.decode('utf-8'))
         if error.decode('utf-8') != "":
@@ -86,7 +86,7 @@ class Tester:
     def diff_files(file1, file2, tmp):
         command = "diff " + file1 + " " + file2
         with open(tmp, "wb") as out:
-            subprocess.Popen(command.split(), stdout=out, stderr=None).communicate()
+            subprocess.Popen(command.split(), stdout=out, stderr=None, shell=is_windows()).communicate()
         with open(tmp, "r") as out:
             lines = out.readlines()
         os.remove(tmp)
@@ -186,7 +186,7 @@ class Tester:
     def _command_jv(self, model, data, variant, prs_jv, special, dataSpecial):
         print()
 
-        cmd = "java -cp " + solver.cp + (" AbsCon " if special else " org.xcsp.modeler.Compiler ")
+        cmd = "java -cp " + solver.cp + (" ace " if special else " org.xcsp.modeler.Compiler ")
         cmd += "problems.generators." + prs_jv if prs_jv else self.dir_pbs_jv + model
         if self.data_jv(data):
             if dataSpecial:
@@ -226,22 +226,22 @@ class Tester:
                 shutil.move(self.name_xml, self.xml_path_py())
                 if mode == 1:  # comparison with jv
                     self.execute_compiler("JvCSP", self._command_jv(model, data, variant, prs_jv, special, dataSpecial))
-                    if os.name != 'nt':
+                    if not is_windows():
                         self.xml_indent(self.name_xml)
                     shutil.move(self.name_xml, self.xml_path_jv())
-                    if os.name != 'nt':
+                    if not is_windows():
                         self.check()
                 elif mode == 2:  # comparison with recorded XCSP files
-                    if os.name != 'nt':
-                        # shutil.copy(self.dir_xcsp + self.name_xml, self.xml_path_jv())  # we copy the xcsp file in the java dir to simulate a comparison with JvCSP
-                        print("  Comparing PyCSP outcome with the XCSP3 file stored in " + self.dir_xcsp)
-                        self.check(True)
+                    # if not is_windows():
+                    # shutil.copy(self.dir_xcsp + self.name_xml, self.xml_path_jv())  # we copy the xcsp file in the java dir to simulate a comparison with JvCSP
+                    print("  Comparing PyCSP outcome with the XCSP3 file stored in " + self.dir_xcsp)
+                    self.check(True)
                 else:
                     with open(self.xml_path_py(), "r") as f:
                         for line in f.readlines():
                             print(COLOR_PY + line[0:-1])
-                #  TODO replace rm -rf by os.remove() and check all system commands
-                if os.name != 'nt':
+                # TODO replace rm -rf by os.remove() and check all system commands
+                if not is_windows():
                     os.system("rm -rf *.*~")  # for removing the temporary files
 
     def check(self, xcsp=False):
@@ -251,6 +251,9 @@ class Tester:
         # LZMA decompress
         xml_lzma = None
         if os.path.isfile(xml_to_compare + ".lzma"):
+            if is_windows():
+                print("  Not comparing because of LZMA on windows")
+                return None
             xml_lzma = xml_to_compare + ".lzma"
             import lzma
             with lzma.open(xml_lzma) as f:
@@ -258,16 +261,16 @@ class Tester:
                 fp = open(xml_to_compare, "wb")
                 fp.write(data)
                 fp.close()
+
             print("Decompress " + xml_lzma + " done.")
 
         # Show differences    
         if not os.path.isfile(self.xml_path_py()) or not os.path.isfile(xml_to_compare):
-            print("error: files not found " + self.xml_path_py() + " or " + xml_to_compare)
+            warning("at least 1 file not found among " + self.xml_path_py() + " and " + xml_to_compare)
             self.counters["err"] += 1
             if waiting:
                 input("Press Enter to continue...")
         else:
-
             lines = self.diff_files(self.xml_path_py(), xml_to_compare, self.tmpDiff)
             if len(lines) == 0:
                 print("  => No difference for " + self.name_xml)
