@@ -1,13 +1,17 @@
 import types
 from collections import deque, namedtuple
 
-from pycsp3.classes.entities import Node, TypeNode
+from pycsp3.classes.entities import Node, TypeNode, ECtr, EMetaCtr
 from pycsp3.classes.main.constraints import (
-    ScalarProduct, PartialConstraint, ConstraintSum, ConstraintElement, ConstraintElementMatrix, ConstraintInstantiation, ECtr, auxiliary)
-from pycsp3.classes.main.variables import Variable, VariableInteger, NotVariable
+    ScalarProduct, PartialConstraint, ConstraintSum, ConstraintElement, ConstraintElementMatrix,
+    ConstraintInstantiation,
+    auxiliary, global_indirection, manage_global_indirection)
+from pycsp3.classes.main.variables import Variable, VariableInteger
 from pycsp3.libs.forbiddenfruit import curse
 from pycsp3.tools.inspector import checkType
-from pycsp3.tools.utilities import flatten, is_containing, unique_type_in, is_1d_tuple, is_1d_list, is_2d_list, is_matrix, ANY, error_if
+from pycsp3.tools.utilities import flatten, is_containing, unique_type_in, is_1d_tuple, is_1d_list, is_matrix, ANY, warning, error_if, error
+
+from pycsp3 import functions
 
 queue_in = deque()  # To store partial constraints when using the IN operator
 
@@ -15,7 +19,7 @@ unsafe_cache = False  # see for example Pic since the table is released as it oc
 
 
 def cursing():
-    def _dict_add(self, other):  # for being able to merge dictionaries
+    def _dict_add(self, other):  # for being able to merge dictionaries (to be removed when python 3.9 will be adopted)
         if isinstance(other, dict):
             d = self.copy()
             d.update(other)
@@ -39,10 +43,14 @@ def cursing():
     # def _list_rmul(self, other):
     #     return _list_mul(other, self)
 
-
     def _tuple_contains(self, other):
         if not OpOverrider.activated:
             return self.__contains__(other)
+        if isinstance(other, Node):
+            other = auxiliary().replace_node(other)
+        if isinstance(other, PartialConstraint):
+            queue_in.append((list(self), other))
+            return True
         if is_containing(other, Variable) and len(self) > 0 and isinstance(self[0], (tuple, int)):
             queue_in.append((list(self), other))
             return True
@@ -54,6 +62,8 @@ def cursing():
     def _list_contains(self, other):  # for being able to use 'in' when expressing extension constraints
         if not OpOverrider.activated:
             return self.__contains__(other)
+        if isinstance(other, Node):
+            other = auxiliary().replace_node(other)
         if isinstance(other, types.GeneratorType):
             other = list(other)
         if is_containing(other, Variable) and len(self) > 0 and isinstance(self[0], (list, tuple, int)):
@@ -71,6 +81,8 @@ def cursing():
     def _set_contains(self, other):  # for being able to use 'in' when expressing intension/extension constraints
         if not OpOverrider.activated:
             return self.__contains__(other)
+        if isinstance(other, Node):
+            other = auxiliary().replace_node(other)
         if isinstance(other, types.GeneratorType):
             other = list(other)
         tself = unique_type_in(self)
@@ -111,6 +123,11 @@ def cursing():
         if isinstance(other, PartialConstraint):
             queue_in.append((self, other))
             return True
+        if isinstance(other, Node):
+            other = auxiliary().replace_node(other)
+            queue_in.append((self, other))
+            return True
+
         return range.__contains__(self, other)
 
     def _enumerate_contains(self, other):
@@ -148,15 +165,16 @@ class OpOverrider:
         ListVar.__ne__ = OpOverrider.__ne__lv
         ListVar.__getitem__ = OpOverrider.__getitem__lv
         ListInt.__getitem__ = OpOverrider.__getitem__li
-        ListInt.__contains__ = OpOverrider.__contains__li
+        # ListInt.__contains__ = OpOverrider.__contains__li
 
-        Variable.__eq__ = Node.__eq__ = OpOverrider.__eq__
-        Variable.__ne__ = Node.__ne__ = OpOverrider.__ne__
+        EMetaCtr.__eq__ = Variable.__eq__ = Node.__eq__ = OpOverrider.__eq__
+        EMetaCtr.__ne__ = Variable.__ne__ = Node.__ne__ = OpOverrider.__ne__
         Variable.__lt__ = Node.__lt__ = OpOverrider.__lt__
         Variable.__le__ = Node.__le__ = OpOverrider.__le__
         Variable.__ge__ = Node.__ge__ = OpOverrider.__ge__
         Variable.__gt__ = Node.__gt__ = OpOverrider.__gt__
 
+        Variable.__neg__ = Node.__neg__ = OpOverrider.__neg__
         Variable.__add__ = Node.__add__ = OpOverrider.__add__
         Variable.__radd__ = Node.__radd__ = OpOverrider.__radd__
         Variable.__sub__ = Node.__sub__ = OpOverrider.__sub__
@@ -168,10 +186,10 @@ class OpOverrider:
         Variable.__floordiv__ = Node.__floordiv__ = OpOverrider.__floordiv__
         Variable.__rfloordiv__ = Node.__rfloordiv__ = OpOverrider.__rfloordiv__
 
-        Variable.__and__ = Node.__and__ = OpOverrider.__and__
-        Variable.__or__ = Node.__or__ = OpOverrider.__or__
-        Variable.__invert__ = Node.__invert__ = OpOverrider.__invert__
-        Variable.__xor__ = Node.__xor__ = OpOverrider.__xor__
+        ECtr.__and__ = EMetaCtr.__and__ = Variable.__and__ = Node.__and__ = OpOverrider.__and__
+        ECtr.__or__ = EMetaCtr.__or__ = Variable.__or__ = Node.__or__ = OpOverrider.__or__
+        ECtr.__invert__ = Node.__invert__ = OpOverrider.__invert__  # we keep __invert__ for Variable
+        ECtr.__xor__ = EMetaCtr.__xor__ = Variable.__xor__ = Node.__xor__ = OpOverrider.__xor__
 
     @staticmethod
     def disable():
@@ -181,15 +199,18 @@ class OpOverrider:
         ListVar.__ne__ = list.__ne__
         ListVar.__getitem__ = list.__getitem__
         ListInt.__getitem__ = list.__getitem__
-        ListInt.__contains__ = list.__contains__
+        # ListInt.__contains__ = list.__contains__
 
-        Variable.__eq__ = Node.__eq__ = object.__eq__
-        Variable.__ne__ = Node.__ne__ = object.__ne__
+        EMetaCtr.__eq__ = Node.__eq__ = object.__eq__
+        Variable.__eq__ = Variable.eq__safe  # TODO are there other methods in the same situation?
+
+        EMetaCtr.__ne__ = Variable.__ne__ = Node.__ne__ = object.__ne__
         Variable.__lt__ = Node.__lt__ = object.__lt__
         Variable.__le__ = Node.__le__ = object.__le__
         Variable.__ge__ = Node.__ge__ = object.__ge__
         Variable.__gt__ = Node.__gt__ = object.__gt__
 
+        Variable.__neg__ = Node.__neg__ = None
         Variable.__add__ = Node.__add__ = None
         Variable.__radd__ = Node.__radd__ = None
         Variable.__sub__ = Node.__sub__ = None
@@ -201,13 +222,25 @@ class OpOverrider:
         Variable.__floordiv__ = Node.__floordiv__ = None
         Variable.__rfloordiv__ = Node.__rfloordiv__ = None
 
-        Variable.__and__ = Node.__and__ = None
-        Variable.__or__ = Node.__or__ = None
-        Variable.__invert__ = Node.__invert__ = None
-        Variable.__xor__ = Node.__xor__ = None
+        ECtr.__and__ = EMetaCtr.__and__ = Variable.__and__ = Node.__and__ = None
+        ECtr.__or__ = EMetaCtr.__or__ = Variable.__or__ = Node.__or__ = None
+        ECtr.__invert__ = Node.__invert__ = None  # we keep __invert__ for Variable
+        ECtr.__xor__ = EMetaCtr.__xor__ = Variable.__xor__ = Node.__xor__ = None
 
         return OpOverrider
 
+    @staticmethod
+    def eq_protected(v1, v2):
+        if isinstance(v1, list) and isinstance(v2, list):
+            return len(v1) == len(v2) and all(OpOverrider.eq_protected(v1[i], v2[i]) for i in range(len(v1)))
+        if type(v1) != type(v2):
+            return False
+        if isinstance(v1, Variable):
+            return Variable.eq__safe(v1, v2)
+        if isinstance(v1, Node):
+            return Node.eq__safe(v1, v2)
+        return v1 == v2
+
     @staticmethod
     def execute(arg):
         OpOverrider.enable()
@@ -218,7 +251,7 @@ class OpOverrider:
         index = slice(None, None, None) if indexes[dimension] == ANY else indexes[dimension]
         if isinstance(index, int):
             if isinstance(t, list):
-                t = t[index]  #  to keep the shape (dimensions), we need to do that
+                t = t[index]  # to keep the shape (dimensions), we need to do that
             else:
                 return t
         elif isinstance(index, slice):
@@ -233,17 +266,41 @@ class OpOverrider:
                 t = OpOverrider.project_recursive(t, indexes, dimension + 1)
         return t
 
+    def __neg__(self):
+        return Node.build(TypeNode.NEG, self)
+
     def __add__(self, other):
+        if isinstance(other, ScalarProduct):
+            other = PartialConstraint(ConstraintSum(other.variables, other.coeffs, None))
         if isinstance(other, PartialConstraint):
-            return PartialConstraint.combine_partial_objects(self, TypeNode.ADD, other)
+            if isinstance(other.constraint, ConstraintSum) and isinstance(self, Node) and all(s.type == TypeNode.VAR for s in self.sons):
+                if self.type == TypeNode.ADD:
+                    for s in self.sons:
+                        other = other + s.sons
+                    return other
+                if self.type == TypeNode.SUB:
+                    return other + self.sons[0].sons - self.sons[1].sons
+            return PartialConstraint.combine_partial_objects(self, TypeNode.ADD, other) if isinstance(other.constraint, ConstraintSum) else other + self
         return Node.build(TypeNode.ADD, self, other)
 
     def __radd__(self, other):
         return Node.build(TypeNode.ADD, other, self)
 
     def __sub__(self, other):
+        if isinstance(other, ScalarProduct):
+            other = PartialConstraint(ConstraintSum(other.variables, other.coeffs, None))
         if isinstance(other, PartialConstraint):
-            return PartialConstraint.combine_partial_objects(self, TypeNode.SUB, other)
+            if isinstance(other.constraint, ConstraintSum) and isinstance(self, Node) and all(s.type == TypeNode.VAR for s in self.sons):
+                if self.type == TypeNode.ADD:
+                    for s in self.sons:
+                        other = other - s.sons
+                    other.constraint.revert_coeffs()
+                    return other
+                if self.type == TypeNode.SUB:
+                    other = other - self.sons[0].sons + self.sons[1].sons
+                    other.constraint.revert_coeffs()
+                    return other
+            return PartialConstraint.combine_partial_objects(self, TypeNode.SUB, other) if isinstance(other.constraint, ConstraintSum) else -(other - self)
         return Node.build(TypeNode.SUB, self, other)
 
     def __rsub__(self, other):
@@ -292,13 +349,21 @@ class OpOverrider:
         return other.__lt__(self) if isinstance(other, (PartialConstraint, ScalarProduct)) else Node.build(TypeNode.GT, self, other)
 
     def __eq__(self, other):
-        if self is None or other is None:
+        res = manage_global_indirection(self, other)
+        if res is None:
+            return functions.Iff(self, other)
+        self, other = res
+        if self is None or other is None:  # we must not write None in (self,other) because recursive behaviour
             return object.__eq__(self, other)
         if isinstance(other, int) and other == 0 and isinstance(self, Node) and self.type == TypeNode.DIST:  # we simplify the expression
             return Node.build(TypeNode.EQ, self.sons[0], self.sons[1])
         return other.__eq__(self) if isinstance(other, (PartialConstraint, ScalarProduct)) else Node.build(TypeNode.EQ, self, other)
 
     def __ne__(self, other):
+        res = manage_global_indirection(self, other)
+        if res is None:
+            return functions.Xor(self, other)  # TODO: is it always appropriate?
+        self, other = res
         if self is None or other is None:
             return object.__ne__(self, other)
         if isinstance(other, int) and other == 0 and isinstance(self, Node) and self.type == TypeNode.DIST:  # we simplify the expression
@@ -306,16 +371,43 @@ class OpOverrider:
         return other.__ne__(self) if isinstance(other, (PartialConstraint, ScalarProduct)) else Node.build(TypeNode.NE, self, other)
 
     def __or__(self, other):
-        return object.__or__(self, other) if None in {self, other} else Node.disjunction(self, other)
+        if isinstance(other, bool):
+            return self if other is False else True
+        res = manage_global_indirection(self, other)
+        if res is None:
+            return functions.Or(self, other)
+        self, other = res
+        if self is None or other is None:
+            return object.__or__(self, other)
+        return Node.disjunction(self, other)
 
     def __and__(self, other):
-        return object.__and__(self, other) if None in {self, other} else Node.conjunction(self, other)
+        if isinstance(other, bool):
+            return self if other is True else False
+        res = manage_global_indirection(self, other)
+        if res is None:
+            return functions.And(self, other)
+        self, other = res
+        if self is None or other is None:
+            return object.__and__(self, other)
+        return Node.conjunction(self, other)
 
     def __invert__(self):
-        return NotVariable(self) if isinstance(self, VariableInteger) else Node.build(TypeNode.NOT, self)
+        if isinstance(self, ECtr):
+            gi = global_indirection(self.constraint)
+            if gi is None:
+                return functions.Not(self)
+            self = gi
+        return Variable.__invert__(self) if isinstance(self, VariableInteger) else Node.build(TypeNode.NOT, self)
 
     def __xor__(self, other):
-        return object.__xor__(self, other) if None in {self, other} else Node.build(TypeNode.XOR, self, other)
+        res = manage_global_indirection(self, other)
+        if res is None:
+            return functions.Xor(self, other)
+        self, other = res
+        if self is None or other is None:
+            return object.__xor__(self, other)
+        return Node.build(TypeNode.XOR, self, other)
 
     def __eq__lv(self, other):  # lv for ListVar
         def Instantiation(*, variables, values):
@@ -341,10 +433,21 @@ class OpOverrider:
     def __getitem__lv(self, indexes):
         if isinstance(indexes, PartialConstraint):
             indexes = auxiliary().replace_partial_constraint(indexes)
+        elif isinstance(indexes, Node):
+            res = indexes.var_val_if_binary_type(TypeNode.ADD)
+            if res is not None and res[1] == 0:  # in case we had x+0 or 0+x, we replace by x
+                indexes = res[0]
+            else:
+                res = indexes.var_val_if_binary_type(TypeNode.MUL)
+                if res is not None and res[1] == 1:  # in case we had x*1 or 1*x, we replace by x
+                    indexes = res[0]
+                else:
+                    # we force the domain of the aux variable with the parameter indexing
+                    indexes = auxiliary().replace_node(indexes, indexing=range(len(self)))
         if isinstance(indexes, Variable):
             return PartialConstraint(ConstraintElement(self, indexes))
         if isinstance(indexes, tuple) and len(indexes) > 0:
-            indexes = auxiliary().replace_partial_constraints(list(indexes))
+            indexes = auxiliary().replace_nodes_and_partial_constraints(list(indexes), nodes_too=True)
             if any(isinstance(i, Variable) for i in indexes):  # this must be a constraint Element-Matrix
                 assert is_matrix(self) and len(indexes) == 2, "A matrix is expected, with two indexes"
                 if all(isinstance(i, Variable) for i in indexes):
@@ -373,7 +476,7 @@ class OpOverrider:
         if isinstance(indexes, Variable):
             return PartialConstraint(ConstraintElement(self, indexes))
         if isinstance(indexes, tuple) and len(indexes) > 0:
-            indexes = auxiliary().replace_partial_constraints(list(indexes))
+            indexes = auxiliary().replace_nodes_and_partial_constraints(list(indexes))
             if any(isinstance(i, Variable) for i in indexes):  # this must be a constraint Element-Matrix
                 assert is_matrix(self) and len(indexes) == 2, "A matrix is expected, with two indexes"
                 if all(isinstance(i, Variable) for i in indexes):
@@ -396,11 +499,11 @@ class OpOverrider:
         except TypeError:
             return result
 
-    def __contains__li(self, other):
-        if is_containing(other, Variable) and len(self) > 0 and isinstance(self[0], (tuple, int)):
-            queue_in.append((self, other))
-            return True
-        return list.__contains__(self, other)
+    # def __contains__li(self, other):
+    #     if is_containing(other, Variable) and len(self) > 0 and isinstance(self[0], (tuple, int)):
+    #         queue_in.append((self, other))
+    #         return True
+    #     return list.__contains__(self, other)
 
 
 class ListInt(list):
@@ -428,6 +531,17 @@ class ListInt(list):
     def __rmul__(self, other):
         return ListInt.__mul__(other, self)
 
+    def __contains__(self, other):
+        if is_containing(other, Variable) and len(self) > 0:
+            if isinstance(self[0], (tuple, int)):
+                queue_in.append((self, other))
+                return True
+            if isinstance(other, list) and isinstance(self[0], list):  # TODO more precise test? and/or make a warning (this should be tuples instead of lists)
+                queue_in.append(([tuple(t) for t in self], other))
+                return True
+                # error("It seems that you build tables whose elements are lists instead of tuples: " + str(other) + " in " + str(self))
+        return list.__contains__(self, other)
+
 
 def columns(m):
     assert is_matrix(m), "columns() can only be called on matrices"
@@ -468,14 +582,26 @@ class ListVar(list):
     def columns(self):
         return columns(self)
 
+    def around(self, i, j):
+        assert is_matrix(self), "calling around should be made on a 2-dimensional array"
+        n, m = len(self), len(self[i])
+        assert 0 <= i < n and 0 <= j < m
+        return ListVar(self[i + k][j + l] for k in [-1, 0, 1] for l in [-1, 0, 1] if 0 <= i + k < n and 0 <= j + l < m and (k, l) != (0, 0))
+
+    def cross(self, i, j):
+        assert is_matrix(self), "calling cross should be made on a 2-dimensional array"
+        n, m = len(self), len(self[i])
+        assert 0 <= i < n and 0 <= j < m
+        return ListVar([self[i][j]] + [self[k][l] for k, l in [(i, j - 1), (i, j + 1), (i - 1, j), (i + 1, j)] if 0 <= k < n and 0 <= l < m])
+
 
 def convert_to_namedtuples(obj):
-    def with_only_alphanumeric_keys(obj):
+    def with_only_alphanumeric_keys(obj):  # alphanum or '_'
         if isinstance(obj, dict):
-            if any(not k.isalnum() for k in obj.keys()):
+            if any(not k.isidentifier() for k in obj.keys()):
                 return False
             return all(with_only_alphanumeric_keys(v) for v in obj.values())
-        if isinstance(obj,(int,str)):
+        if isinstance(obj, (int, str)):
             return True
         try:
             iter(obj)
@@ -509,7 +635,7 @@ def convert_to_namedtuples(obj):
         return obj
 
     if not with_only_alphanumeric_keys(obj):
-        print("\nWarning: some key of some dictionary involved in the data is not alphanumeric, so no conversion to named tuples is performed\n")
+        warning("some key of some dictionary involved in the data is not alphanumeric, so no conversion to named tuples is performed\n")
         return obj  # not possible to make the conversion in that case
     return recursive_convert_to_namedtuples(obj)
 
