@@ -3,9 +3,9 @@ import os
 import sys
 
 from pycsp3.dashboard import options
-from pycsp3.tools.utilities import flatten
+from pycsp3.tools.utilities import flatten, is_windows
 
-if not os.name == 'nt':
+if not is_windows():
     import readline
 
 fileToTrace = [arg for arg in sys.argv if arg.endswith(".py") and "compiler.py" not in arg] + ["<stdin>"]
@@ -13,7 +13,8 @@ fileToTrace = [arg for arg in sys.argv if arg.endswith(".py") and "compiler.py"
 
 def _global_value_of(variable):
     stack = list(reversed(inspect.stack(context=1)))
-    frames = [frame for frame in stack if frame.filename in fileToTrace]
+    frames = [frame for frame in stack for trace in fileToTrace if trace in frame.filename]
+    # frames = [frame for frame in stack if frame.filename in fileToTrace]
     if len(frames) == 0:
         return None
     globs = frames[0].frame.f_globals
@@ -35,39 +36,34 @@ def is_continued_line(line):
     return isinstance(line, str) and line.strip().endswith('\\')
 
 
-def is_correct_frame(frame, f):
-    return frame.function == f or frame.function == "__init__" and f in frame.code_context[0]
-
-
 # Starts at the end of the function and runs it up until the function name is found
 def browse_code_bottom_to_top(lines, function_name):
-    codes = []
+    code = []
     function_name_found = False
     for line in lines:
         if function_name_found is False:
-            codes.append(line)
+            code.append(line)
             if function_name in line and not is_comment_line(line):
                 function_name_found = True
         else:
             if is_continued_line(line) or is_empty_line(line) or is_comment_line(line):
-                codes.append(line)
+                code.append(line)
             else:
                 break
-    return codes
+    return code
 
 
 def browse_code_top_to_bottom(lines, function_name):
-    codes = []
+    code = []
     function_name_found = False
     level = 0
     for line in lines:
         if function_name in line and not is_comment_line(line):
             function_name_found = True
-            codes.append(line)
+            code.append(line)
             line = line.split(function_name)[1]
         else:
-            codes.append(line)
-
+            code.append(line)
         if function_name_found:
             for letter in line:
                 if letter in {'(', '['}:
@@ -76,50 +72,37 @@ def browse_code_top_to_bottom(lines, function_name):
                     level = level - 1
         if level == 0:
             break
-    return list(reversed(codes))
+    return list(reversed(code))
 
 
 def _extract_correct_frame(function_name):
-    stack = list(reversed(inspect.stack(context=1)))
-    frame = [(i - 1, stack[i - 1]) for i, frame in enumerate(stack) if is_correct_frame(frame, function_name) and i > 0][0]  # Get the correct frame
-    return frame
-
-
-def _extract_code_index_last_line(frame, function_name):
-    lines = list(reversed(frame.code_context[:frame.index + 1]))
-    return browse_code_bottom_to_top(lines, function_name)
-
-
-def _extract_code_index_first_line(frame, function_name):
-    if function_name == "Var" or function_name == "VarArray":
-        lines = list(reversed(frame.code_context[:frame.index + 1]))
-        return browse_code_bottom_to_top(lines, function_name)
-    else:
-        lines = list(frame.code_context[frame.index:])
-        return browse_code_top_to_bottom(lines, function_name)
+    for i, frame_info in enumerate(reversed(inspect.stack(context=1))):
+        if i > 0 and (frame_info.function == function_name or frame_info.function == "__init__" and function_name in frame_info.code_context[0]):
+            return i - 1, previous  # Get the correct frame
+        previous = frame_info
+    return None
 
 
 def _extract_code(function_name):
-    frame = _extract_correct_frame(function_name)  # getting the good frame
+    index, frame_info = _extract_correct_frame(function_name)  # getting the good frame
 
-    # Console case
-    if frame[1].filename == "<stdin>":
-        if os.name == 'nt':
-            assert os.name != 'nt', "Console mode is not available on Windows"
-        lines = reversed([readline.get_history_item(i + 1) for i in range(readline.get_current_history_length())])
+    if frame_info.filename == "<stdin>":  # Console case
+        if is_windows():
+            assert False, "Console mode is not available on Windows"
+        lines = reversed(list(readline.get_history_item(i + 1) for i in range(readline.get_current_history_length())))
         return browse_code_bottom_to_top(lines, function_name)
 
     # The index of the line in the stack of the inspector changes between python 3.7 and 3.8:
-    #  In 3.8 the index is the line where the function name appears
-    #  In 3.7 and lower versions, it is the line at the end of the function
-    #  So the algorithms are completely different
-    frame = list(reversed(inspect.stack(context=100)))[frame[0]]
-    if sys.version_info[1] == 8:
-        codes = _extract_code_index_first_line(frame, function_name)
+    # In 3.8 the index is the line where the function name appears
+    # In 3.7 and lower versions, it is the line at the end of the function
+    # So the algorithms are completely different
+    frame_info = list(reversed(inspect.stack(context=2000)))[index]  # TODO how to avoid this constant?
+    if sys.version_info[1] >= 8 and function_name != "Var" and function_name != "VarArray":  # why special cases for Var and VarArray?
+        lines = list(frame_info.code_context[frame_info.index:])
+        return browse_code_top_to_bottom(lines, function_name)
     else:
-        codes = _extract_code_index_last_line(frame, function_name)
-
-    return codes
+        lines = reversed(frame_info.code_context[:frame_info.index + 1])
+        return browse_code_bottom_to_top(lines, function_name)
 
 
 # returns a pair (left,right) of positions of the first occurrence of the specified separators in the line, or None
@@ -244,7 +227,7 @@ def comments_and_tags_of_parameters_of(*, function_name, args):
                         tags2[i1].append("")
                         new_line += '$'
                     else:
-                        assert level != 0
+                        assert level != 0, line + " " + str(level)
                         new_line += '_'
                 else:
                     new_line += c
@@ -312,8 +295,8 @@ def extract_declaration_for(function_name):
 def checkType(obj, allowed_types, message=""):
     if options.checker == "none":
         return True
-    if options.checker == "fast" and isinstance(obj, (list, tuple, set)) and len(obj) > 100:
-        obj = obj[:1]
+    if options.checker == "fast" and isinstance(obj, (list, tuple, set, frozenset)) and len(obj) > 100:
+        obj = obj[:1]  # TODO problem here: sets cannot be indexed
     allowed_types = (allowed_types,) if not isinstance(allowed_types, tuple) else allowed_types
     for allowedType in allowed_types:
         if not isinstance(allowedType, list):
@@ -326,7 +309,7 @@ def checkType(obj, allowed_types, message=""):
             else:
                 return True
     if message == "":
-        message = "Wrong type for " + str(obj) + " (allowable types: " + str(allowed_types) + ")\n"    
+        message = "Wrong type for " + str(obj) + " (allowable types: " + str(allowed_types) + ")\n"
     raise TypeError(message)
 
     # stack = inspect.stack(context=1)
@@ -374,4 +357,4 @@ def checkType(obj, allowed_types, message=""):
     #     # doc = docstringOf(stack[1].function)
     #     # s += doc if doc is not None else "\n\tNo python docstring for this function"
 
-      # s)
+    # s)
