@@ -1,3 +1,4 @@
+import os
 import sys
 import types
 from collections.abc import Iterable
@@ -163,21 +164,18 @@ def matrix_to_string(m):
     # return "\n" + "\n".join(["\t(" + ",".join([str(v) for v in t]) + ")" for t in m]) + "\n"
 
 
-def transitions_to_string(ts):
-    return "".join(["(" + q1 + "," + str(v) + "," + q2 + ")" for (q1, v, q2) in ts])
-
-
 def table_to_string(table, restricting_domains=None, *, parallel=False):
     def _tuple_to_string(t):
         return "(" + ",".join(
-            str(v) if isinstance(v, (int)) else 
+            str(v) if isinstance(v, (int)) else
             str(set(v)) if isinstance(v, tuple) else
             conditions.inside(v).str_tuple() if isinstance(v, range) else
-            v if isinstance(v, (str)) else 
-            "*" if v == ANY else v.str_tuple() 
+            v if isinstance(v, (str)) else
+            "*" if v == ANY else v.str_tuple()
             for v in t) + ")"
 
-    if not parallel or len(table) < 100000:
+    LIMIT = 100000  # hard coding
+    if not parallel or len(table) < LIMIT:
         s = []
         previous = ""
         for t in table:  # table is assumed to be sorted (adding an assert?) ; only distinct tuples are kept
@@ -188,7 +186,7 @@ def table_to_string(table, restricting_domains=None, *, parallel=False):
                 previous = t
         return "".join(s)
     else:
-        print("Parallel creation of a table of size: " + str(len(table)))
+        print("Creation of a table of size: " + str(len(table)) + (" in parallel" if parallel and len(table) >= LIMIT else ""))
         n_threads = cpu_count()
         size = len(table) // n_threads
         pool = Pool(n_threads)
@@ -256,20 +254,38 @@ def display_constraints(ctr_entities, separator=""):
                 print(separator + str(ce.constraint))
 
 
-PURPLE, BLUE, GREEN, ORANGE, RED, WHITE, WHITE_BOLD, UNDERLINE = '\033[95m', '\033[94m', '\033[92m', '\033[93m', '\033[91m', '\033[0m', '\033[1m', '\033[4m'
+def is_windows():
+    return os.name == 'nt'
+
+
+def _proxy_color(s):
+    return "" if is_windows() else s
+
+
+PURPLE, BLUE, GREEN, ORANGE, RED, WHITE, WHITE_BOLD, UNDERLINE = _proxy_color('\033[95m'), _proxy_color('\033[94m'), _proxy_color('\033[92m'), _proxy_color(
+    '\033[93m'), _proxy_color('\033[91m'), _proxy_color('\033[0m'), _proxy_color('\033[1m'), _proxy_color('\033[4m')
+
+
+def string_color(s, start, final=WHITE):
+    return start + s + final
+
+
+class Error:
+    errorOccurrence = False
 
 
 def warning(message):
-    print("  " + ORANGE + "Warning: " + WHITE + message)
+    print("\n  " + ORANGE + "Warning: " + WHITE + message)
 
 
 def error(s):
+    Error.errorOccurrence = True
     print("\n\t" + RED + "ERROR: " + WHITE, s, "\n")
     print("\t\t(add option -ev to your command if you want to see the trace of the error)\n")
     if options.ev:
         raise TypeError(s)
     else:
-        sys.exit(0)
+        sys.exit(1)
 
 
 def error_if(test, s):
