@@ -2,13 +2,14 @@ from collections import OrderedDict
 
 from lxml import etree
 
-from pycsp3.classes.auxiliary.ptypes import TypeFramework, TypeXML, TypeVar, TypeCtr, TypeCtrArg
-from pycsp3.classes.entities import Entity, EVar, EVarArray, ECtr, EObjective, EAnnotation, EGroup, \
+from pycsp3.classes.auxiliary.ptypes import TypeFramework, TypeConditionOperator, TypeXML, TypeVar, TypeCtr, TypeCtrArg
+from pycsp3.classes.entities import Entity, EVar, EVarArray, ECtr, EMetaCtr, EObjective, EAnnotation, EGroup, \
     EBlock, ESlide, EIfThenElse, EToGather, EToSatisfy, CtrEntities, VarEntities, ObjEntities, AnnEntities
 from pycsp3.classes.main.constraints import ConstraintIntension
 from pycsp3.dashboard import options
 from pycsp3.tools.compactor import compact
 from pycsp3.tools.slider import _identify_slide
+from pycsp3.classes.auxiliary.conditions import Condition
 
 SIZE_LIMIT_FOR_USING_AS = 12  # when building domains of variables of arrays of variables (and using the attribute 'as')
 
@@ -87,7 +88,7 @@ def _variables():
     return elt
 
 
-def _argument(elt, arg, key, value):
+def _argument(elt, arg, key, value, change_element_value=False):
     assert value is not None
     if arg.lifted is True:  # TODO do we have an example? (3-tuples for attributes in this case?)
         for i, l in enumerate(value):
@@ -96,7 +97,14 @@ def _argument(elt, arg, key, value):
                 subelt.set(str(att[2]), str(att[3]))
             elt.append(subelt)
     else:
-        elt.append(_element(key, attributes=arg.attributes, text=None if isinstance(value, list) and len(value) == 1 and value[0] is None else value))
+        # the first part (if) will be removed in the medium term (we will systematically use the XML <condition>)
+        if change_element_value and key == TypeCtrArg.CONDITION and ((isinstance(value, Condition) and value.operator == TypeConditionOperator.EQ)
+                                                                     or (isinstance(value, str) and value.startswith("(eq,"))):
+            v = str(value)
+            elt.append(_element(TypeCtrArg.VALUE, attributes=arg.attributes, text=v[v.index(',') + 1:-1]))
+        else:
+            v = None if isinstance(value, list) and len(value) == 1 and value[0] is None else value
+            elt.append(_element(key, attributes=arg.attributes, text=v))
 
 
 def _constraint(entity, *, possible_simplified_form=False):
@@ -111,8 +119,9 @@ def _constraint(entity, *, possible_simplified_form=False):
     if len(arguments) == 1 and not arguments[0].lifted and (possible_simplified_form or arguments[0].name == TypeCtrArg.LIST):
         _text(elt, arguments[0].content)
     else:
+        b = c.name == TypeCtr.ELEMENT
         for arg in arguments:
-            _argument(elt, arg, arg.name, arg.content)
+            _argument(elt, arg, arg.name, arg.content, change_element_value=b)
     return elt
 
 
@@ -126,8 +135,9 @@ def _constraint_template(group):
         if len(arguments) == 1 and TypeCtrArg.LIST in group.abstraction:
             _text(elt, group.abstraction[TypeCtrArg.LIST])
         else:
+            b = str(first.constraint.name) == "element"
             for k, v in arguments:
-                _argument(elt, first.constraint.arguments[k], k, v)
+                _argument(elt, first.constraint.arguments[k], k, v, change_element_value=b)
         return elt
 
 
@@ -171,8 +181,8 @@ def _constraints_recursive(elt, entity):
     elif isinstance(entity, EBlock):
         if len(entity.entities) != 0:
             son = _constraints_iterative(_element(TypeXML.BLOCK, entity), entity.entities)
-    elif isinstance(entity, EIfThenElse):
-        son = _constraints_iterative(_element(TypeXML.ifThenElse, entity), entity.entities)
+    elif isinstance(entity, EMetaCtr):
+        son = _constraints_iterative(_element(str(entity.id), entity), entity.entities)
     elif isinstance(entity, EToSatisfy):
         _constraints_iterative(elt, entity.entities)
     else:
